# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.rds.v1 import _proxy
from otcextensions.sdk.rds.v1 import backup as _backup
# from otcextensions.sdk.rds.v1 import configuration as _configuration
from otcextensions.sdk.rds.v1 import datastore as _datastore
from otcextensions.sdk.rds.v1 import flavor as _flavor
from otcextensions.sdk.rds.v1 import instance as _instance

PROJECT_ID = '123'
ENDPOINT_OS = 'http://rds.example.com/v1.0'
ENDPOINT_RDS = 'http://rds.example.com/rds/v1'

# RDS requires those headers to be present in the request, to native API
# otherwise 404
RDS_HEADERS = {
    'Content-Type': 'application/json',
    'X-Language': 'en-us'
}

# RDS requires those headers to be present in the request, to OS-compat API
# otherwise 404
OS_HEADERS = {
    'Content-Type': 'application/json',
}


class TestRdsProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestRdsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)
        self.session.get_project_id = mock.Mock(return_value=PROJECT_ID)

        self.proxy.get_os_endpoint = mock.Mock(
            return_value=ENDPOINT_OS
        )
        self.proxy.get_rds_endpoint = mock.Mock(
            return_value=ENDPOINT_RDS
        )
        self.session.get_endpoint = mock.Mock(
            return_value=ENDPOINT_RDS
        )

        self.additional_headers = RDS_HEADERS

    def test_datastore_types(self):
        result = self.proxy.datastore_types()

        self.assertEqual(['MySQL', 'PostgreSQL', 'SQLServer'],
                         list(s.name for s in result))

    def test_datastore_versions(self):
        self.verify_list(
            self.proxy.datastore_versions, _datastore.Datastore,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            method_kwargs={
                'datastore': 'test',
            },
            expected_kwargs={
                'paginated': False,
                'datastore_name': 'test',
                'headers': RDS_HEADERS,
            }
        )

    def test_get_datastore_version(self):
        # TODO(agoncharov) we invoke list, and then process result
        # implement test to mock results
        pass
        # self.verify_list(
        #     self.proxy.get_datastore_version, _datastore.Datastore,
        #     mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
        #     method_kwargs={
        #         'datastore': 'test_ds',
        #         'datastore_version': 'test_ver',
        #     },
        #     paginated=False,
        #     expected_kwargs={
        #         'datastore_name': 'test_ds',
        #         'datastore_version': 'test_ver',
        #         'endpoint_override': ENDPOINT_RDS,
        #         'headers': RDS_HEADERS,
        #         'project_id': PROJECT_ID
        #     }
        # )

    def test_flavors(self):
        self.verify_list(
            self.proxy.flavors, _flavor.Flavor,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            method_args=["dbId", "regio"],
            expected_kwargs={
                'paginated': False,
                'dbId': 'dbId',
                'region': 'regio',
                'headers': RDS_HEADERS
            },
            expected_args=[]
        )

    def test_get_flavor(self):
        self.verify_get(
            self.proxy.get_flavor,
            _flavor.Flavor,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
            expected_kwargs={
                'headers': RDS_HEADERS
            }
        )

    # def test_find_flavor(self):
    #     self._verify(
    #         'otcextensions.sdk.sdk_proxy.Proxy._find',
    #         self.proxy.find_flavor,
    #         method_args=["flavor"],
    #         expected_args=[_flavor.Flavor, "flavor"],
    #         expected_kwargs={
    #             "ignore_missing": True})

    def test_create_instance(self):
        self.verify_create(
            self.proxy.create_instance, _instance.Instance,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'instance': 'test',
                'name': 'some_name',
                'headers': RDS_HEADERS
            }
        )

    def test_delete_instance(self):
        self.verify_delete(
            self.proxy.delete_instance,
            _instance.Instance, True,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
            expected_kwargs={
                'headers': RDS_HEADERS,
            }
        )

    def test_update_instance(self):
        self._verify(
            'otcextensions.sdk.sdk_proxy.Proxy._update',
            self.proxy.update_instance,
            method_args=['INSTANCE'],
            method_kwargs={'test': 't'},
            expected_args=[_instance.Instance],
            expected_kwargs={
                'test': 't',
                'instance': 'INSTANCE',
                'headers': RDS_HEADERS
            }
        )

    def test_get_instance(self):
        self.verify_get(
            self.proxy.get_instance,
            _instance.Instance,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
            expected_kwargs={
                'headers': RDS_HEADERS
            }
        )

    def test_find_instance(self):
        self._verify(
            'otcextensions.sdk.sdk_proxy.Proxy._find',
            self.proxy.find_instance,
            method_args=["instance"],
            expected_args=[_instance.Instance, "instance"],
            expected_kwargs={
                'headers': RDS_HEADERS,
                "ignore_missing": True})

    def test_instances(self):
        self.verify_list(
            self.proxy.instances, _instance.Instance,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            expected_kwargs={
                'paginated': False,
                'headers': RDS_HEADERS
            }
        )

    # def test_configurations(self):
    #     self.verify_list(
    #         self.proxy.configurations, _configuration.ConfigurationGroup,
    #         paginated=False,
    #         mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
    #         expected_kwargs={
    #             'headers': RDS_HEADERS
    #         }
    #     )
    #
    # def test_get_configuration(self):
    #     self.verify_get(
    #         self.proxy.get_configuration,
    #         _configuration.ConfigurationGroup,
    #         mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
    #         expected_kwargs={
    #             'headers': RDS_HEADERS
    #         }
    #     )
    #
    # def test_create_configuration(self):
    #     self.verify_create(
    #         self.proxy.create_configuration,
    #         _configuration.ConfigurationGroup,
    #         mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
    #         method_kwargs={
    #             'instance': 'test',
    #             'name': 'some_name'
    #         },
    #         expected_kwargs={
    #             'headers': RDS_HEADERS,
    #             'instance': 'test',
    #             'name': 'some_name'
    #         }
    #     )
    #
    # def test_update_configuration_group(self):
    #     self.verify_update(
    #         self.proxy.update_configuration_group,
    #         _configuration.ConfigurationGroup,
    #         mock_method='otcextensions.sdk.sdk_proxy.Proxy._update',
    #         method_kwargs={
    #             'instance': 'test',
    #             'name': 'some_name'
    #         },
    #         expected_kwargs={
    #             'project_id': PROJECT_ID,
    #             'endpoint_override': ENDPOINT_OS,
    #             'headers': OS_HEADERS,
    #             'instance_id': 'test',
    #             'name': 'some_name'
    #         }
    #     )
    #
    # def test_delete_configuration(self):
    #     self.verify_delete(
    #         self.proxy.delete_configuration,
    #         _configuration.ConfigurationGroup, True,
    #         mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
    #         expected_kwargs={
    #             'headers': RDS_HEADERS,
    #         }
    #     )
    #
    # def test_find_configuration(self):
    #     self._verify(
    #         'otcextensions.sdk.sdk_proxy.Proxy._find',
    #         self.proxy.find_configuration,
    #         method_args=["config"],
    #         expected_args=[_configuration.ConfigurationGroup, "config"],
    #         expected_kwargs={
    #             # 'project_id': PROJECT_ID,
    #             # 'endpoint_override': ENDPOINT_OS,
    #             # 'headers': OS_HEADERS,
    #             "ignore_missing": True})

    def test_backups(self):
        self.verify_list(
            self.proxy.backups, _backup.Backup,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            expected_kwargs={
                'paginated': False,
                'headers': RDS_HEADERS
            }
        )

    def test_create_backup(self):
        self.verify_create(
            self.proxy.create_backup, _backup.Backup,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'headers': RDS_HEADERS,
                'instance': 'test',
                'name': 'some_name'
            }
        )

    def test_delete_backup(self):
        self.verify_delete(
            self.proxy.delete_backup,
            _backup.Backup, False,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
            expected_kwargs={
                'headers': RDS_HEADERS,
            }
        )

    def test_get_backup_policy(self):
        self.verify_get(
            self.proxy.get_backup_policy, _backup.BackupPolicy,
            method_args=[],
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
            method_kwargs={
                'instance': 'id'
            },
            expected_kwargs={
                'headers': RDS_HEADERS,
                'instance_id': 'id',
                'requires_id': False
            }
        )

    def test_set_backup_policy(self):
        # TODO(agoncharov) upstream BaseProxy is renamed to Proxy
        self._verify(
            'otcextensions.sdk.sdk_proxy.Proxy._update',
            self.proxy.set_backup_policy,
            method_args=['POLICY', 'INST_ID'],
            expected_args=[_backup.BackupPolicy, 'POLICY'],
            expected_kwargs={
                'instance_id': 'INST_ID',
                'headers': RDS_HEADERS
            }
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts Service Update Logs v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class ServiceEvents(command.Lister):
    _description = _("List update logs of a real-time service.")
    columns = (
        "Occur Time",
        "Event Type",
        "Event Info",
    )

    def get_parser(self, prog_name):
        parser = super(ServiceEvents, self).get_parser(prog_name)

        parser.add_argument(
            "service",
            metavar="<service>",
            help=_("Service ID or Name."),
        )
        parser.add_argument(
            "--event-type",
            metavar="<event_type>",
            help=_(
                "Type of the event to be filtered. By default, the event "
                "type is not filtered. Options:"
                "\n`normal`: normal events"
                "\n`abnormal`: abnormal events"
            ),
        )
        parser.add_argument(
            "--start-time",
            metavar="<start_time>",
            type=int,
            help=_(
                "Start time of the event to be filtered. The value is "
                "milliseconds."
            ),
        )
        parser.add_argument(
            "--end-time",
            metavar="<end_time>",
            type=int,
            help=_(
                "End time of the event to be filtered. The value is "
                "milliseconds."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Start page of the paging list. Default value: 0"),
        )
        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Maximum number of records returned on each page. "
                "Default value: 1000"
            ),
        )
        parser.add_argument(
            "--sort-by",
            metavar="<sort_by>",
            help=_(
                "Specified sorting field. The default value is occur_time."
            ),
        )
        parser.add_argument(
            "--order",
            metavar="<order>",
            help=_(
                "Sorting mode. The default value is desc. Options:"
                "\n`asc`: ascending order"
                "\n`desc`: descending order"
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        args_list = (
            "end_time",
            "event_type",
            "limit",
            "offset",
            "order",
            "sort_by",
            "start_time",
        )
        query_params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        service = client.find_service(
            parsed_args.service, ignore_missing=False
        )
        data = client.service_events(service.id, **query_params)

        formatters = {
            "Occur Time": cli_utils.UnixTimestampFormatter,
        }

        return (
            self.columns,
            (
                utils.get_item_properties(
                    s, self.columns, formatters=formatters
                )
                for s in data
            ),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import maintenance_window as mw


JSON_DATA = {
    "default": False,
    "seq": 1,
    "begin": "22",
    "end": "02"
}


class TestMW(base.TestCase):

    def test_basic(self):
        sot = mw.MaintenanceWindow()

        self.assertEqual('/instances/maintain-windows', sot.base_path)
        self.assertEqual('maintain_windows', sot.resources_key)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = mw.MaintenanceWindow(**JSON_DATA)
        self.assertEqual(JSON_DATA['default'], sot.is_default)
        self.assertEqual(int(JSON_DATA['seq']), sot.seq)
        self.assertEqual(JSON_DATA['begin'], sot.begin)
        self.assertEqual(JSON_DATA['end'], sot.end)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class ServiceFlavor(resource.Resource):
    base_path = "/services/specifications"

    resources_key = "specifications"

    allow_list = True

    _query_mapping = resource.QueryParameters(
        "is_personal_cluster",
        "infer_type",
    )

    # Properties
    #: Unique ID of the billing specifications.
    billing_spec = resource.Body("billing_spec")
    #: Specifications description in English.
    display_en = resource.Body("display_en")
    #: Billing item.
    extend_params = resource.Body("extend_params")
    #: Whether the quota exceeds the upper limit. The value true indicates
    #:  that the quota exceeds the upper limit.
    over_quota = resource.Body("over_quota", type=bool)
    #: Whether the flavor is free of charge. The value true indicates
    #:  that the flavor is free of charge.
    is_free = resource.Body("is_free", type=bool)
    #: Whether to enable the specifications. The default value is true.
    #:  If this parameter is set to false, you need to submit a service
    #:  ticket to apply for the specifications.
    is_open = resource.Body("is_open", type=bool)
    #: Specifications status.
    #: \n`normal`: The specifications are normal.
    #: \n`sellout`: The specifications cannot be used to deploy services
    #:  because they are sold out.
    spec_status = resource.Body("spec_status")
    #: Type of the model to which the specifications apply.
    #:  `Empty`: indicates a model generated by the user.
    source_type = resource.Body("source_type")
    #: Unique specifications ID.
    specification = resource.Body("specification")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class ServiceMonitor(resource.Resource):
    base_path = "/services/%(service_id)s/monitor"

    resources_key = "monitors"

    allow_list = True

    _query_mapping = resource.QueryParameters(
        "node_id",
    )

    # Properties
    #: Model ID.
    model_id = resource.Body("model_id")
    #: Model name.
    model_name = resource.Body("model_name")
    #: Model Version.
    model_version = resource.Body("model_version")
    #: Total number of model instance calls.
    invocation_times = resource.Body("invocation_times", type=int)
    #: Number of failed model instance calls.
    failed_times = resource.Body("failed_times", type=int)
    #: Number of used CPUs.
    cpu_core_usage = resource.Body("cpu_core_usage", type=float)
    #: Total number of CPUs.
    cpu_core_total = resource.Body("cpu_core_total", type=float)
    #: Used memory, in MB.
    cpu_memory_usage = resource.Body("cpu_memory_usage", type=int)
    #: Total memory, in MB.
    cpu_memory_total = resource.Body("cpu_memory_total", type=int)
    #: Number used GPUs.
    gpu_usage = resource.Body("gpu_usage", type=float)
    #: Total number of GPUs.
    gpu_total = resource.Body("gpu_total", type=float)
    #: Service ID.
    service_id = resource.URI("service_id")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import json

from openstack import exceptions
from openstack import resource


class SpecificationSpec(resource.Resource):
    #: Minimum CPU.
    min_cpu = resource.Body("min_cpu")
    #: Minimum GPU.
    min_gpu = resource.Body("min_gpu")
    #: Minimum memory capacity.
    min_memory = resource.Body("min_memory")


class HealthSpec(resource.Resource):
    #: After an instance is started, a health check starts after seconds
    #:  configured in initial_delay_seconds.
    initial_delay_seconds = resource.Body("initial_delay_seconds")
    #: Request protocol of the health check interface.
    protocol = resource.Body("protocol")
    #: Health check timeout.
    timeout_seconds = resource.Body("timeout_seconds")
    #: URL of the health check interface.
    url = resource.Body("url")


class OutputParamsSpec(resource.Resource):
    #: Properties of an object element in JSON Schema.
    properties = resource.Body("properties", type=dict)
    # Required.
    required = resource.Body("required", type=list)
    #: Type in JSON Schema, which can be object.
    type = resource.Body("type")


class InputParamsSpec(resource.Resource):
    #: Properties of an object element in JSON Schema.
    properties = resource.Body("properties", type=dict)
    #: Type in JSON Schema, which can be object.
    type = resource.Body("type")


class ApisSpec(resource.Resource):
    #: Content Type.
    content_type = resource.Body("content_type")
    #: Input parameters in apis, described in JSON Schema format.
    input_params = resource.Body("input_params", type=InputParamsSpec)
    #: Request method.
    method = resource.Body("method")
    #: Output parameters in apis, described in JSON Schema format.
    output_params = resource.Body("output_params", type=OutputParamsSpec)
    #: Request protocol.
    protocol = resource.Body("protocol")
    #: Inference request URL.
    url = resource.Body("url")


class TemplateInputsSpec(resource.Resource):
    #: Template input path, which can be an OBS file path or OBS directory
    #:  path.
    input = resource.Body("input")
    #: Input item ID, which is obtained from the template details.
    input_id = resource.Body("input_id")


class TemplateSpec(resource.Resource):
    #: Input and output mode.
    infer_format = resource.Body("infer_format")
    #: ID of the used template.
    template_id = resource.Body("template_id")
    #: Template input configuration, specifying the source path for
    #:  configuring a model.
    template_inputs = resource.Body(
        "template_inputs", type=list, list_type=TemplateInputsSpec
    )


class DocSpec(resource.Resource):
    #: Document name, which must start with a letter.
    doc_name = resource.Body("doc_name")
    #: HTTP(S) link of the document.
    doc_url = resource.Body("doc_url")


class MetricSpec(resource.Resource):
    #: Accuracy.
    accuracy = resource.Body("accuracy", type=float)
    #: F1 score.
    f1 = resource.Body("f1", type=float)
    #: Precision.
    precision = resource.Body("precision", type=float)
    #: Recall.
    recall = resource.Body("recall", type=float)


class PackageSpec(resource.Resource):
    #: Name of a dependency package.
    package_name = resource.Body("package_name")
    #: Version of a dependency package.
    package_version = resource.Body("package_version")
    #: Version restriction.
    restraint = resource.Body("restraint")


class DependencySpec(resource.Resource):
    #: Installation mode.
    installer = resource.Body("installer")
    #: Collection of dependency packages.
    packages = resource.Body("packages", type=list, list_type=PackageSpec)


class ParamsSpec(resource.Resource):
    #: This parameter is optional when param_type is set to int or float.
    max = resource.Body("max", type=int)
    #: Request method.
    method = resource.Body("method")
    #: This parameter is optional when param_type is set to int or float.
    min = resource.Body("min", type=int)
    #: Parameter description.
    param_desc = resource.Body("param_desc")
    #: Parameter name.
    param_name = resource.Body("param_name")
    #: Parameter type.
    param_type = resource.Body("param_type")
    #: Request protocol.
    protocol = resource.Body("protocol")
    #: API URL.
    url = resource.Body("url")


class Model(resource.Resource):
    base_path = "/models"

    resources_key = "models"

    paginated = False

    # capabilities
    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "name",
        "model_version",
        "status",
        "model_type",
        "not_model_type",
        "description",
        "offset",
        "marker",
        "limit",
        "sort_by",
        "order",
        "workspace_id",
        name="model_name",
        status="model_status",
    )

    # Properties
    #: AI project.
    ai_project = resource.Body("ai_project")
    #: All input and output apis parameter information of a model, which is
    #:  obtained from the model preview.
    apis = resource.Body("apis", type=list, list_type=ApisSpec)
    #: Model configurations.
    config = resource.Body("config")
    #: Time when a model is created, in milliseconds calculated from 1970.
    created_at = resource.Body("create_at", type=int)
    #: Package required for inference code and model.
    dependencies = resource.Body(
        "dependencies", type=list, list_type=DependencySpec
    )
    #: Model description.
    description = resource.Body("description")
    #: OBS path for storing the execution code. The name of the
    #:  execution code file is fixed to *customize_service.py*.
    execution_code = resource.Body("execution_code")
    #: Model health check interface information.
    health = resource.Body("health", type=HealthSpec)
    #: Character string converted from the final model configuration file.
    initial_config = resource.Body("initial_config")
    #: image path generated after model packaging.
    image_address = resource.Body("image_address")
    #: Collection of input parameters of a model.
    input_params = resource.Body(
        "input_params", type=list, list_type=ParamsSpec
    )
    #: Supported service type for deployment.
    install_type = resource.Body("install_type", type=list)
    #: Whether a model can be published to the marketplace.
    is_publishable = resource.Body("publishable_flag", type=bool)
    #: Whether a model is subscribed from the marketplace.
    is_subscribed = resource.Body("market_flag", type=bool)
    #: Whether a model can be tuned.
    is_tunable = resource.Body("tunable", type=bool)
    #: Model label map.
    labels_map = resource.Body("labels_map", type=dict)
    #: Model algorithm type. The value can be *predict_analysis*,
    #:  *object_detection*, or *image_classification*.
    model_algorithm = resource.Body("model_algorithm")
    #: List of model description documents.
    model_docs = resource.Body("model_docs", type=list, list_type=DocSpec)
    #: Model ID.
    model_id = resource.Body("model_id", alternate_id=True)
    #: Model label array.
    model_labels = resource.Body("model_labels", type=list)
    #: Model precision, which is read from the configuration file.
    model_metrics = resource.Body("model_metrics")
    #: Model name.
    model_name = resource.Body("model_name")
    #: Model size, in bytes.
    model_size = resource.Body("model_size", type=int)
    #: Model source.
    model_source = resource.Body("model_source")
    #: Model status.
    model_status = resource.Body("model_status")
    #: Model type. The value can be *TensorFlow*, *MXNet*, *Spark_MLlib*,
    #:  *Scikit_Learn*, *XGBoost*, *Image*, or *PyTorch*.
    model_type = resource.Body("model_type")
    #: Model version.
    model_version = resource.Body("model_version")
    #: Model name.
    name = resource.Body("name", alias="model_name")
    #: Collection of output parameters of a model.
    output_params = resource.Body(
        "output_params", type=list, list_type=ParamsSpec
    )
    #: User to which a model belongs.
    owner_id = resource.Body("owner")
    #: Project to which a model belongs.
    project_id = resource.Body("project")
    #: Model runtime environment.
    runtime = resource.Body("runtime")
    #: Download address of the model schema file.
    schema_doc = resource.Body("schema_doc")
    #: ID of the source training job.
    source_job_id = resource.Body("source_job_id")
    #: Version of the source training job.
    source_job_version = resource.Body("source_job_version")
    #: OBS path where the model is located or the SWR image location.
    source_location = resource.Body("source_location")
    #: Model source type. If a model is deployed through ExeML, the value is
    #:  auto. If a model is deployed through a training job or an OBS model
    #:  file, this parameter is left blank.
    source_type = resource.Body("source_type")
    #: Minimum model deployment specifications.
    specification = resource.Body("specification", type=SpecificationSpec)
    #: Template configuration items.
    template = resource.Body("template", type=TemplateSpec)
    #: Model status.
    status = resource.Body("status", alias="model_status")
    #: Tenant to which a model belongs.
    tenant_id = resource.Body("tenant")
    #: Model version.
    version = resource.Body("version", alias="model_version")
    #: ID of the workspace to which a service belongs.
    workspace_id = resource.Body("workspace_id")

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data
        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.
        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body

        exceptions.raise_from_response(response, error_message=error_message)

        if has_body:
            try:
                body = response.json()
                if self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]
                for key in (
                    "model_metrics",
                    "config",
                    "apis",
                    "output_params",
                    "input_params",
                ):
                    if key in body:
                        if isinstance(body[key], str):
                            body[key] = json.loads(body[key])
                        elif key in (
                            "output_params",
                            "input_params",
                        ) and isinstance(body[key], list):
                            for param in body[key]:
                                if isinstance(param.get("param_desc"), str):
                                    param["param_desc"] = json.loads(
                                        param["param_desc"]
                                    )

                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                body.pop("self", None)
                body_attrs = self._consume_body_attrs(body)
                if self._allow_unknown_attrs_in_body:
                    body_attrs.update(body)
                    self._unknown_attrs_in_body.update(body)
                elif self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body
                    )

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.dis.v2 import _proxy
from otcextensions.sdk.dis.v2 import stream
from otcextensions.sdk.dis.v2 import app
from otcextensions.sdk.dis.v2 import checkpoint
from otcextensions.sdk.dis.v2 import dump_task

from openstack.tests.unit import test_proxy_base


class TestDisProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestDisProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestStream(TestDisProxy):

    def test_stream_create(self):
        self.verify_create(self.proxy.create_stream, stream.Stream,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_stream_delete(self):
        self.verify_delete(self.proxy.delete_stream,
                           stream.Stream, True)

    def test_stream_get(self):
        self.verify_get(self.proxy.get_stream, stream.Stream)

    def test_streams(self):
        self.verify_list(self.proxy.streams, stream.Stream,
                         method_kwargs={'x': 1, 'y': 2},
                         expected_kwargs={'x': 1, 'y': 2})

    def test_stream_partition_update(self):
        self._verify('openstack.proxy.Proxy._update',
                     self.proxy.update_stream_partition,
                     method_args=['test-stream', 10],
                     expected_args=[stream.Stream, 'test-stream'],
                     expected_kwargs={
                         'stream_name': 'test-stream',
                         'target_partition_count': 10
                     })


class TestApp(TestDisProxy):

    def test_app_create(self):
        self.verify_create(self.proxy.create_app, app.App,
                           method_kwargs={'name': 'id'},
                           expected_kwargs={'name': 'id'})

    def test_app_delete(self):
        self.verify_delete(self.proxy.delete_app, app.App, True)

    def test_app_get(self):
        self.verify_get(self.proxy.get_app, app.App)

    def test_apps(self):
        self.verify_list(self.proxy.apps, app.App)

    def test_app_consumptions(self):
        self.verify_list(self.proxy.app_consumptions, app.AppConsumption,
                         method_kwargs={
                             'app_name': 'test-app',
                             'stream_name': 'test-stream',
                         },
                         expected_kwargs={
                             'app_name': 'test-app',
                             'stream_name': 'test-stream',
                             'checkpoint_type': 'LAST_READ'
                         })


class TestCheckpoint(TestDisProxy):
    def test_checkpoint_create(self):
        self.verify_create(self.proxy.create_checkpoint,
                           checkpoint.Checkpoint,
                           method_kwargs={'arg1': 'a1'},
                           expected_kwargs={
                               'arg1': 'a1',
                               'checkpoint_type': 'LAST_READ'
                           })

    def test_checkpoint_delete(self):
        self._verify(
            'otcextensions.sdk.dis.v2.checkpoint.Checkpoint.delete_checkpoint',
            self.proxy.delete_checkpoint,
            method_args=['test-stream', 'test-app'],
            expected_args=[self.proxy],
            expected_kwargs={
                'stream_name': 'test-stream',
                'app_name': 'test-app',
                'checkpoint_type': 'LAST_READ'
            }
        )

    def test_checkpoint_get(self):
        self._verify(
            'otcextensions.sdk.dis.v2.checkpoint.Checkpoint.get_checkpoint',
            self.proxy.get_checkpoint,
            method_args=['test-stream', 'test-app', 'partition300'],
            expected_args=[self.proxy],
            expected_kwargs={
                'stream_name': 'test-stream',
                'app_name': 'test-app',
                'partition_id': 'partition300',
                'checkpoint_type': 'LAST_READ'
            }
        )


class TestDumpTask(TestDisProxy):
    def test_dump_task_create(self):
        self.verify_create(self.proxy.create_dump_task,
                           dump_task.DumpTask,
                           method_kwargs={
                               'stream_name': 'test-stream',
                               'arg1': 'test-arg'
                           },
                           expected_kwargs={
                               'arg1': 'test-arg',
                               'uri_stream_name': 'test-stream',
                           })

    def test_dump_task_delete(self):
        self.verify_delete(self.proxy.delete_dump_task,
                           dump_task.DumpTask, False,
                           method_args=['test-stream', 'test-dump-task'],
                           expected_kwargs={
                               'uri_stream_name': 'test-stream',
                               'ignore_missing': False
                           },
                           expected_args=['test-dump-task'])

    def test_dump_task_get(self):
        self.verify_get(self.proxy.get_dump_task,
                        dump_task.DumpTask,
                        method_args=['test-stream', 'test-dump-task'],
                        expected_kwargs={'uri_stream_name': 'test-stream'},
                        expected_args=['test-dump-task'])

    def test_dump_tasks(self):
        self.verify_list(self.proxy.dump_tasks,
                         dump_task.DumpTask,
                         method_kwargs={'stream_name': 'test-stream'},
                         expected_kwargs={
                             'uri_stream_name': 'test-stream'
                         })

    def test_dump_task_start(self):
        self._verify(
            'otcextensions.sdk.dis.v2.dump_task.DumpTask._action',
            self.proxy.start_dump_task,
            method_args=['test-stream'],
            expected_args=[self.proxy, 'test-stream', 'start']
        )

    def test_dump_task_pause(self):
        self._verify(
            'otcextensions.sdk.dis.v2.dump_task.DumpTask._action',
            self.proxy.pause_dump_task,
            method_args=['test-stream'],
            expected_args=[self.proxy, 'test-stream', 'stop']
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from openstack import _log
from openstack import exceptions

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestDataKey(base.BaseFunctionalTest):

    def setUp(self):
        super(TestDataKey, self).setUp()
        # self.cmk = self.conn.kms.find_key(alias='sdk_test_key1')
        self.cmk = self.conn.kms.create_key(
            key_alias=uuid.uuid4().hex
        )

    def tearDown(self):
        try:
            if self.cmk:
                key = self.cmk
                if key.id:
                    self.conn.kms.schedule_key_deletion(key, 7)
        except exceptions.SDKException as e:
            self.warning = _logger.warning('Got exception during '
                                           'clearing resources %s' % e.message)
        super(TestDataKey, self).tearDown()

    def test_dek(self):

        cmk = self.cmk

        dek = self.conn.kms.create_datakey(
            cmk=cmk,
            datakey_length=512,
            encryption_context={"a": "b", "c": "d"})
        self.assertIsNotNone(dek.plain_text)

        dek2 = self.conn.kms.create_datakey_wo_plain(
            cmk=cmk,
            datakey_length=512,
            encryption_context={"a": "b", "c": "d"})
        self.assertIsNone(dek2.plain_text)

        not_encrypted_value = dek.plain_text
        self.conn.kms.encrypt_datakey(dek)

        encrypted_value = dek.cipher_text

        self.assertIsNotNone(encrypted_value)
        self.assertNotEqual(not_encrypted_value, encrypted_value)
        _logger.debug('encrypted DEK = %s' % encrypted_value)

        decrypt_key = self.conn.kms.decrypt_datakey(
            cmk=cmk,
            cipher_text=encrypted_value,
            datakey_cipher_length=dek.datakey_cipher_length)
        decrypted_value = decrypt_key.plain_text
        self.assertEqual(not_encrypted_value, decrypted_value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DrillServers(resource.Resource):
    #: Properties
    #: Protected instance ID of drill server
    protected_instance = resource.Body('protected_instance')
    #: Drill server ID
    drill_server_id = resource.Body('drill_server_id')


class DRDrill(resource.Resource):
    """SDRS Disaster recovery Drill Resource"""
    resource_key = 'disaster_recovery_drill'
    resources_key = 'disaster_recovery_drills'
    base_path = '/disaster-recovery-drills'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'drill_vpc_id', 'limit', 'marker', 'name',
        'offset', 'server_group_id', 'status')

    #: Properties
    #: Creation time
    created_at = resource.Body('created_at')
    #: Drill servers information
    drill_servers = resource.Body('drill_servers',
                                  type=list,
                                  list_type=DrillServers)
    #: DR drill VPC ID
    drill_vpc_id = resource.Body('drill_vpc_id')
    #: Job ID
    job_id = resource.Body('job_id')
    #: DR drill ID
    id = resource.Body('id')
    #: DR drill name
    name = resource.Body('name')
    #: Protection group ID
    server_group_id = resource.Body('server_group_id')
    #: DR drill status
    status = resource.Body('status')
    #: Update time
    updated_at = resource.Body('updated_at')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class PublicIPInfo(resource.Resource):
    #: Properties
    #: Specifies the ID of the EIP that uses the bandwidth.
    ip_version = resource.Body('publicip_id', type=int)
    #: Specifies the IP address version.
    publicip_id = resource.Body('publicip_id')
    #: Specifies the obtained EIP if only IPv4 EIPs are available.
    publicip_address = resource.Body('publicip_address')
    #: Specifies the EIP type.
    publicip_type = resource.Body('publicip_type', default='5_bgp')


class Bandwidth(resource.Resource):
    resources_key = 'bandwidths'
    resource_key = 'bandwidth'
    base_path = '/%(version)s/%(project_id)s/bandwidths'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters()

    # Properties
    #: Specifies the bandwidth name.
    #: *Type: dict*
    name = resource.Body('name', type=str)
    #: Specifies the bandwidth size.
    #: *Type: dict*
    size = resource.Body('size', type=int)
    #: Specifies whether the bandwidth is shared or dedicated.
    share_type = resource.Body('share_type', type=str)
    #: Specifies the project ID.
    publicip_info = resource.Body('publicip_info', type=list,
                                  list_type=PublicIPInfo)
    #: Specifies the project ID.
    project_id = resource.URI('project_id')
    #: Specifies the bandwidth type.
    bandwidth_type = resource.Body('bandwidth_type', type=str, default='share')
    #: Specifies that the bandwidth is billed by bandwidth.
    charge_mode = resource.Body('charge_mode', type=str)
    #: Specifies the bill information.
    billing_info = resource.Body('billing_info', type=str)
    #: Specifies the enterprise project ID.
    enterprise_project_id = resource.Body('enterprise_project_id', type=str)
    #: Specifies the bandwidth status.
    status = resource.Body('status', type=str)
    #: Specifies the time (UTC) when the bandwidth is created.
    created_at = resource.Body('created_at', type=str)
    #: Specifies the time (UTC) when the bandwidth is updated.
    updated_at = resource.Body('updated_at', type=str)

    def add_eip_to_bandwidth(self, session, version,
                             publicip_info, project_id):
        """Method to add an EIP to shared bandwidth.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param list publicip_info: List from dictionaries which describes eips.
        """
        path = self.base_path % {'project_id': project_id, 'version': version}
        print(path)
        url = utils.urljoin(path, self.id, 'insert')
        body = {'bandwidth': {'publicip_info': publicip_info}}
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def remove_eip_from_bandwidth(self, session, project_id,
                                  version, **attrs):
        """Method to remove an EIP from shared bandwidth.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param dict attrs: Describes eip info.
        """
        path = self.base_path % {'project_id': project_id, 'version': version}
        url = utils.urljoin(path, self.id, 'remove')
        body = {'bandwidth': attrs}
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def update_bandwidth(self, session, project_id, version, **attrs):
        """Method to update shared bandwidth.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param kwargs attrs: Dictionary to udate bandwidth
        """
        path = self.base_path % {'project_id': project_id, 'version': version}
        url = utils.urljoin(path, self.id)
        body = {'bandwidth': attrs}
        response = session.put(url, json=body)
        return self._to_object(session, response)

    def _to_object(self, session, response):
        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        microversion = self._get_microversion(session)
        self.microversion = microversion
        self._translate_response(response, has_body=has_body)
        if self.has_body and self.create_returns_body is False:
            return self.fetch(session)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import mock

from otcextensions.osclient.identity.v3 import custom_role
from otcextensions.tests.unit.osclient.identity.v3 import fakes


class TestListIdentityCustomRoles(fakes.TestIdentity):

    objects = fakes.FakeIdentityCustomRole.create_multiple(3)

    column_list_headers = (
        'ID',
        'Name',
        'Description',
        'Domain ID',
        'References',
        'Catalog',
        'Display name',
        'Type',
        'Created At',
        'Updated At',
    )

    columns = (
        'id',
        'name',
        'description',
        'domain_id',
        'references',
        'catalog',
        'display_name',
        'type',
        'created_at',
        'updated_at',
    )

    data = []

    for o in objects:
        data.append(
            (o.id, o.name, o.description, o.domain_id, o.references,
             o.catalog, o.display_name, o.type, o.created_at, o.updated_at))

    def setUp(self):
        super(TestListIdentityCustomRoles, self).setUp()

        self.cmd = custom_role.ListCustomRoles(self.app, None)

        self.client.custom_roles = mock.Mock()
        self.client.api_mock = self.client.custom_roles

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--page', '1',
            '--per_page', '2'
        ]

        verifylist = [
            ('page', '1'),
            ('per_page', '2'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            page='1', per_page='2'
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import random

from openstackclient.tests.functional import base


class TestRdsConfiguration(base.TestCase):
    """Functional tests for RDS Configurations. """

    def test_list(self):
        json_output = json.loads(self.openstack(
            'rds configuration list -f json '
        ))
        self.assertIsNotNone(json_output)

    def test_show(self):
        json_output = json.loads(self.openstack(
            'rds configuration list -f json'
        ))

        json_output = json.loads(self.openstack(
            'rds configuration show {cfg} -f json'.format(
                cfg=json_output[0]['ID'])
        ))

        self.assertIsNotNone(json_output['id'])

    def test_create(self):
        name = 'osc-test-config-' + format(random.randint(1, 2**16), '04x')
        json_output = json.loads(self.openstack(
            'rds configuration create '
            '{name} '
            '--datastore-type MySQL '
            '--datastore-version 5.7 '
            '-f json'.format(
                name=name)
        ))
        self.assertIsNotNone(json_output)

        self.openstack(
            'rds configuration delete '
            '{name} '.format(
                name=name)
        )
        self.assertTrue(True)

    def test_create_with_value(self):
        name = 'osc-test-config-' + format(random.randint(1, 2**16), '04x')
        json_output = json.loads(self.openstack(
            'rds configuration create '
            '{name} '
            '--datastore-type MySQL '
            '--datastore-version 5.7 '
            '--value max_connections=10 '
            '--value autocommit=OFF '
            '-f json'.format(
                name=name)
        ))
        self.assertIsNotNone(json_output)

        self.openstack(
            'rds configuration delete '
            '{name} '.format(
                name=name)
        )
        self.assertTrue(True)

    def test_config_param_list(self):
        json_output = json.loads(self.openstack(
            'rds configuration list -f json'
        ))

        json_output = json.loads(self.openstack(
            'rds configuration parameter list '
            '{cfg} '
            '-f json'.format(
                cfg=json_output[0]['ID'])
        ))
        self.assertIsNotNone(json_output)

    def test_config_set(self):
        name = 'osc-test-config-' + format(random.randint(1, 2**16), '04x')
        json_output = json.loads(self.openstack(
            'rds configuration create '
            '{name} '
            '--datastore-type MySQL '
            '--datastore-version 5.7 '
            '-f json'.format(
                name=name)
        ))
        self.assertIsNotNone(json_output)

        self.openstack(
            'rds configuration set '
            '{name} '
            '--value max_connections=10 '
            '--value autocommit=OFF'.format(name=name)
        )
        self.assertTrue(True)

        self.openstack(
            'rds configuration delete '
            '{name} '.format(
                name=name)
        )
        self.assertTrue(True)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import warnings
from urllib.parse import urlparse

from openstack import exceptions
from openstack import service_description

from otcextensions.sdk.vpc.v1 import _proxy


class VpcService(service_description.ServiceDescription):
    """The VPC service."""

    supported_versions = {
        '1': _proxy.Proxy
    }

    def _make_proxy(self, instance):
        """Create a Proxy for the service in question.

        :param instance:
          The `openstack.connection.Connection` we're working with.
        """
        config = instance.config

        # First, check to see if we've got config that matches what we
        # understand in the SDK.
        version_string = '1'
        endpoint_override = config.get_endpoint(self.service_type)
        ep = config.get_service_catalog().url_for(
            service_type=self.service_type,
            region_name=config.region_name)
        _base = urlparse(ep)

        epo = '%(scheme)s://%(base)s' % {
            'scheme': _base.scheme,
            'base': _base.netloc
        }

        if epo and not endpoint_override:
            endpoint_override = epo

        # If the user doesn't give a version in config, but we only support
        # one version, then just use that version.
        if not version_string and len(self.supported_versions) == 1:
            version_string = list(self.supported_versions)[0]

        proxy_obj = None
        if endpoint_override and version_string and self.supported_versions:
            # Both endpoint override and version_string are set, we don't
            # need to do discovery - just trust the user.
            proxy_class = self.supported_versions.get(version_string[0])
            if proxy_class:
                proxy_obj = config.get_session_client(
                    self.service_type,
                    constructor=proxy_class,
                )
                proxy_obj.endpoint_override = endpoint_override
                proxy_obj.additional_headers = {
                    'Content-Type': 'application/json'}
            else:
                warnings.warn(
                    "The configured version, {version} for service"
                    " {service_type} is not known or supported by"
                    " openstacksdk. The resulting Proxy object will only"
                    " have direct passthrough REST capabilities.".format(
                        version=version_string,
                        service_type=self.service_type),
                    category=exceptions.UnsupportedServiceVersion)
        elif endpoint_override and self.supported_versions:
            temp_adapter = config.get_session_client(
                self.service_type
            )
            api_version = temp_adapter.get_endpoint_data().api_version
            proxy_class = self.supported_versions.get(str(api_version[0]))
            if proxy_class:
                proxy_obj = config.get_session_client(
                    self.service_type,
                    constructor=proxy_class,
                )
            else:
                warnings.warn(
                    "Service {service_type} has an endpoint override set"
                    " but the version discovered at that endpoint, {version}"
                    " is not supported by openstacksdk. The resulting Proxy"
                    " object will only have direct passthrough REST"
                    " capabilities.".format(
                        version=api_version,
                        service_type=self.service_type),
                    category=exceptions.UnsupportedServiceVersion)

        if proxy_obj:
            return proxy_obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from openstack import exceptions


class ObsInfoSpec(resource.Resource):
    bucket_name = resource.Body('bucket_name')
    file_prefix_name = resource.Body('file_prefix_name')
    is_obs_created = resource.Body('is_obs_created', type=bool)
    compress_type = resource.Body('compress_type')
    is_sort_by_service = resource.Body('is_sort_by_service', type=bool)


class LtsSpec(resource.Resource):
    is_lts_enabled = resource.Body('is_lts_enabled', type=bool)
    log_group_name = resource.Body('log_group_name')
    log_topic_name = resource.Body('log_topic_name')


class Tracker(resource.Resource):
    base_path = '/trackers'
    resources_key = 'trackers'
    allow_list = True
    allow_create = True
    allow_delete = True
    allow_commit = True
    requires_id = False

    _query_mapping = resource.QueryParameters('tracker_name')
    id = resource.Body('id')
    create_time = resource.Body('create_time')
    lts = resource.Body('lts', type=LtsSpec)
    tracker_type = resource.Body('tracker_type')
    domain_id = resource.Body('domain_id')
    project_id = resource.Body('project_id')
    tracker_name = resource.Body('tracker_name')
    status = resource.Body('status')
    detail = resource.Body('detail')
    obs_info = resource.Body('obs_info', type=ObsInfoSpec)
    group_id = resource.Body('group_id')
    stream_id = resource.Body('stream_id')
    is_lts_enabled = resource.Body('is_lts_enabled')

    def delete_tracker(self, session):
        path = f'{self.base_path}?tracker_name={self.tracker_name}'
        response = session.delete(path)
        exceptions.raise_from_response(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from collections import namedtuple

import mock

from otcextensions.osclient.mrs.v1 import cluster
from otcextensions.tests.unit.osclient.mrs.v1 import fakes


class TestCluster(fakes.TestMrs):

    def setUp(self):
        super(TestCluster, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeCluster.create_one()

        flat_data = cluster._flatten_cluster(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['status'],
            flat_data['region'],
            flat_data['cluster_type'],
            flat_data['availability_zone'],
            flat_data['version'],
            flat_data['tags'],
            flat_data['created_at'],
            flat_data['updated_at'],
            flat_data['billing_type'],
            flat_data['vpc'],
            flat_data['master_node_size'],
            flat_data['core_node_size'],
            flat_data['external_ip'],
            flat_data['internal_ip'],
            flat_data['master_num'],
            flat_data['core_num'],
            flat_data['core_node_size'],
            flat_data['component_list'],
            flat_data['deployment_id'],
            flat_data['instance_id'],
            flat_data['vnc'],
            flat_data['project_id'],
            flat_data['volume_size'],
            flat_data['volume_type'],
            flat_data['subnet_id'],
            flat_data['subnet_name'],
            flat_data['security_group_id'],
            flat_data['non_master_security_group_id'],
            flat_data['safe_mode'],
            flat_data['key'],
            flat_data['master_ip'],
            flat_data['preffered_private_ip'],
            flat_data['charging_start_time'],
            flat_data['task_node_groups'],
            flat_data['node_groups'],
            flat_data['bootstrap_scripts'],
            flat_data['scale'],
        )

        cmp_data = (
            obj.id,
            obj.name,
            obj.status,
            obj.region,
            obj.cluster_type,
            obj.az_id,
            obj.cluster_version,
            obj.tags,
            cluster._utc_to_timestamp(obj.created_at),
            cluster._utc_to_timestamp(obj.updated_at),
            obj.billing_type,
            obj.vpc,
            obj.master_node_size,
            obj.core_node_size,
            obj.external_ip,
            obj.internal_ip,
            obj.master_num,
            obj.core_num,
            obj.core_node_size,
            obj.component_list,
            obj.deployment_id,
            obj.instance_id,
            obj.vnc,
            obj.project_id,
            obj.volume_size,
            obj.volume_type,
            obj.subnet_id,
            obj.subnet_name,
            obj.security_group_id,
            obj.non_master_security_group_id,
            obj.safe_mode,
            obj.key,
            obj.master_ip,
            obj.preffered_private_ip,
            obj.charging_start_time,
            obj.task_node_groups,
            obj.node_groups,
            obj.bootstrap_scripts,
            obj.scale
        )

        self.assertEqual(data, cmp_data)

    def test_normalize_tags(self):
        tags = ['k1=v1', 'k2', 'k3=']

        verify_result = 'k1*v1,k2,k3'

        result = cluster._normalize_tags(tags)

        self.assertEqual(result, verify_result)

    def test_add_tags_to_cluster_output(self):
        Obj = namedtuple('obj', 'tags')
        obj = Obj(['an=,1=2'])

        column = ()
        data = ()
        verify_column = (
            'tags',
        )
        verify_data = ('key=an, value=\nkey=1, value=2\n'),

        data, column = cluster._add_tags_to_cluster_obj(
            obj, data, column
        )

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)


class TestListCluster(fakes.TestMrs):
    objects = fakes.FakeCluster.create_multiple(3)

    columns = ('id', 'name', 'status', 'region',
               'cluster_type', 'availability_zone', 'version')

    data = []

    for s in objects:
        flat_data = cluster._flatten_cluster(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['status'],
            flat_data['region'],
            flat_data['cluster_type'],
            flat_data['availability_zone'],
            flat_data['version'],
        ))

    def setUp(self):
        super(TestListCluster, self).setUp()

        self.cmd = cluster.ListCluster(self.app, None)

        self.client.clusters = mock.Mock()
        self.client.api_mock = self.client.clusters

    def test_default(self):
        arglist = []

        verifylist = []

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowCluster(fakes.TestMrs):
    object = fakes.FakeCluster.create_one()

    columns = (
        'ID',
        'name',
        'status',
        'region',
        'cluster_type',
        'availability_zone',
        'version',
        'created_at',
        'updated_at',
        'billing_type',
        'vpc',
        'core_node_flavor',
        'external_ip',
        'internal_ip',
        'master_num',
        'core_num',
        'master_node_size',
        'core_node_size',
        'deployment_id',
        'component_list',
        'instance_id',
        'vnc',
        'project_id',
        'volume_size',
        'volume_type',
        'subnet_id',
        'subnet_name',
        'security_group_id',
        'non_master_security_group_id',
        'safe_mode',
        'key',
        'master_ip',
        'preffered_private_ip',
        'charging_start_time',
        'task_node_groups',
        'node_groups',
        'bootstrap_scripts',
        'scale'
    )

    flat_data = cluster._flatten_cluster(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['status'],
        flat_data['region'],
        flat_data['cluster_type'],
        flat_data['availability_zone'],
        flat_data['version'],
        flat_data['created_at'],
        flat_data['updated_at'],
        flat_data['billing_type'],
        flat_data['vpc'],
        flat_data['core_node_flavor'],
        flat_data['external_ip'],
        flat_data['internal_ip'],
        flat_data['master_num'],
        flat_data['core_num'],
        flat_data['master_node_size'],
        flat_data['core_node_size'],
        flat_data['deployment_id'],
        flat_data['component_list'],
        flat_data['instance_id'],
        flat_data['vnc'],
        flat_data['project_id'],
        flat_data['volume_size'],
        flat_data['volume_type'],
        flat_data['subnet_id'],
        flat_data['subnet_name'],
        flat_data['security_group_id'],
        flat_data['non_master_security_group_id'],
        flat_data['safe_mode'],
        flat_data['key'],
        flat_data['master_ip'],
        flat_data['preffered_private_ip'],
        flat_data['charging_start_time'],
        flat_data['task_node_groups'],
        flat_data['node_groups'],
        flat_data['bootstrap_scripts'],
        flat_data['scale'],
    )

    def setUp(self):
        super(TestShowCluster, self).setUp()

        self.cmd = cluster.ShowCluster(self.app, None)

        self.client.find_cluster = mock.Mock()

    def test_default(self):
        arglist = [
            'cluster'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_cluster.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster.assert_called_once_with(
            name_or_id='cluster',
            ignore_missing=False, )

        self.data, self.columns = cluster._add_tags_to_cluster_obj(
            self.object,
            self.data,
            self.columns,
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteCluster(fakes.TestMrs):

    def setUp(self):
        super(TestDeleteCluster, self).setUp()

        self.cmd = cluster.DeleteCluster(self.app, None)

        self.client.delete_cluster = mock.Mock()

    def test_delete(self):
        arglist = [
            'cluster'
        ]
        verifylist = []
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_cluster.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                cluster='cluster',
                ignore_missing=False),
        ]

        self.client.delete_cluster.assert_has_calls(delete_calls)
        self.assertEqual(1, self.client.delete_cluster.call_count)


class TestUpdateCluster(fakes.TestMrs):
    object = fakes.FakeCluster.create_one()

    columns = (
        'ID',
        'name',
        'status',
        'region',
        'cluster_type',
        'availability_zone',
        'version',
        'created_at',
        'updated_at',
        'billing_type',
        'vpc',
        'core_node_flavor',
        'external_ip',
        'internal_ip',
        'master_num',
        'core_num',
        'master_node_size',
        'core_node_size',
        'deployment_id',
        'component_list',
        'instance_id',
        'vnc',
        'project_id',
        'volume_size',
        'volume_type',
        'subnet_id',
        'subnet_name',
        'security_group_id',
        'non_master_security_group_id',
        'safe_mode',
        'key',
        'master_ip',
        'preffered_private_ip',
        'charging_start_time',
        'task_node_groups',
        'node_groups',
        'bootstrap_scripts',
        'scale'
    )

    flat_data = cluster._flatten_cluster(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['status'],
        flat_data['region'],
        flat_data['cluster_type'],
        flat_data['availability_zone'],
        flat_data['version'],
        flat_data['created_at'],
        flat_data['updated_at'],
        flat_data['billing_type'],
        flat_data['vpc'],
        flat_data['core_node_flavor'],
        flat_data['external_ip'],
        flat_data['internal_ip'],
        flat_data['master_num'],
        flat_data['core_num'],
        flat_data['master_node_size'],
        flat_data['core_node_size'],
        flat_data['deployment_id'],
        flat_data['component_list'],
        flat_data['instance_id'],
        flat_data['vnc'],
        flat_data['project_id'],
        flat_data['volume_size'],
        flat_data['volume_type'],
        flat_data['subnet_id'],
        flat_data['subnet_name'],
        flat_data['security_group_id'],
        flat_data['non_master_security_group_id'],
        flat_data['safe_mode'],
        flat_data['key'],
        flat_data['master_ip'],
        flat_data['preffered_private_ip'],
        flat_data['charging_start_time'],
        flat_data['task_node_groups'],
        flat_data['node_groups'],
        flat_data['bootstrap_scripts'],
        flat_data['scale'],
    )

    def setUp(self):
        super(TestUpdateCluster, self).setUp()

        self.cmd = cluster.UpdateCluster(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.update_cluster = mock.Mock()

    def test_default(self):
        arglist = [
            'cluster_id',
            '--scale_type', 'scale_in',
            '--instances', '3',
        ]
        verifylist = [
            ('cluster', 'cluster_id'),
            ('scale_type', 'scale_in'),
            ('instances', 3),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_cluster.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_cluster.assert_called_with(
            name_or_id='cluster_id',
            ignore_missing=False)

        self.client.update_cluster.assert_called_once_with(
            cluster=mock.ANY,
            parameters={
                'scale_type': 'scale_in',
                'node_id': 'node_orderadd',
                'instances': 3
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import time

from openstack import exceptions
from openstack import proxy
from otcextensions.sdk.css.v1 import cluster as _cluster
from otcextensions.sdk.css.v1 import cluster_image as _cluster_image
from otcextensions.sdk.css.v1 import (
    cluster_upgrade_status as _cluster_upgrade_status
)
from otcextensions.sdk.css.v1 import flavor as _flavor
from otcextensions.sdk.css.v1 import snapshot as _snapshot


class Proxy(proxy.Proxy):
    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Accept': 'application/json',
            'Content-type': 'application/json',
        }

    # ======== Cluster ========
    def clusters(self, **query):
        """List all Clusters.

        :returns: a generator of
            (:class:`~otcextensions.sdk.css.v1.cluster.Cluster`) instances
        """
        if query.get('limit'):
            query.update(paginated=False)
        return self._list(_cluster.Cluster, **query)

    def get_cluster(self, cluster):
        """Get the cluster by UUID

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`

        :returns: instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        """
        return self._get(_cluster.Cluster, cluster)

    def find_cluster(self, name_or_id, ignore_missing=True):
        """Find a single cluster

        :param name_or_id: The name or ID of a CSS cluster
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the cluster does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent cluster.

        :returns:
            One :class:`~otcextensions.sdk.css.v1.cluster.Cluster` or ``None``
        """
        return self._find(
            _cluster.Cluster, name_or_id, ignore_missing=ignore_missing
        )

    def create_cluster(self, **attrs):
        """Create a cluster from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.css.v1.cluster.Cluster`,
            comprised of the properties on the Cluster class.
        :returns: The results of cluster creation
        :rtype: :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        """
        return self._create(_cluster.Cluster, **attrs)

    def restart_cluster(self, cluster):
        """Restart a cluster.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.restart(self)

    def extend_cluster(self, cluster, add_nodes):
        """Scaling Out a Cluster with only Common Nodes

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param add_nodes: Number of common nodes to be scaled out.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.extend(self, add_nodes)

    def extend_cluster_nodes(self, cluster, **attrs):
        """Scaling Out a Cluster with Special Nodes

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param data: Cluster scale-out request data.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._create(
            _cluster.ExtendClusterNodes, cluster_id=cluster.id, **attrs
        )

    def delete_cluster(self, cluster, ignore_missing=True):
        """Delete a cluster

        :param cluster: The value can be the ID of a cluster
             or a :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
             instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent cluster.
        :returns: ``None``
        """
        return self._delete(
            _cluster.Cluster, cluster, ignore_missing=ignore_missing
        )

    def update_cluster_name(self, cluster, new_name):
        """Update cluster name

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param new_name: new name for the CSS cluster
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_name(self, new_name)

    def update_cluster_password(self, cluster, new_password):
        """Update cluster password

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param new_password: new password for the CSS cluster
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_password(self, new_password)

    def update_cluster_flavor(
        self, cluster, new_flavor, node_type=None, check_replica=True
    ):
        """Update cluster flavor

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param check_replica: indicates whether to verify replicas
        :param new_flavor: ID of the new flavor
        :param node_type: the type of node
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_flavor(
            self, new_flavor, node_type, check_replica
        )

    def update_cluster_security_mode(
        self,
        cluster,
        https_enable=None,
        authority_enable=None,
        admin_pwd=None,
    ):
        """Update cluster security mode

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param authority_enable: indicates whether to enable the security mode
        :param admin_pwd: cluster password in security mode
        :param https_enable: indicates whether to enable HTTPS
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_security_mode(
            self, https_enable, authority_enable, admin_pwd
        )

    def update_cluster_security_group(self, cluster, security_group_id):
        """Update cluster security group

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param security_group_id: new security group id for the CSS cluster
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_security_group(self, security_group_id)

    def update_cluster_kernel(
        self,
        cluster,
        target_image_id,
        upgrade_type,
        indices_backup_check,
        agency,
        cluster_load_check=False,
    ):
        """Update cluster kernel

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param target_image_id: ID of the target image version.
        :param upgrade_type: upgrade type
        :param indices_backup_check: indicates whether to perform backup
            verification
        :param agency: agency name
        :param cluster_load_check: indicates whether to verify the load
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.update_kernel(
            self,
            target_image_id,
            upgrade_type,
            indices_backup_check,
            agency,
            cluster_load_check,
        )

    def get_cluster_version_upgrades(self, cluster, upgrade_type):
        """Get cluster version upgrade info

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param upgrade_type: version type
        :returns: image info list
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._get(
            _cluster_image.ClusterImage,
            cluster_id=cluster.id,
            upgrade_type=upgrade_type,
            requires_id=False,
        )

    def scale_in_cluster(self, cluster, nodes):
        """Scale in a cluster by removing specified nodes

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param nodes: list of node id
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.scale_in(self, nodes)

    def scale_in_cluster_by_node_type(self, cluster, nodes):
        """Remove instances of specific types and reduce instance
            storage capacity in a cluster

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param nodes: type and quantity of nodes Type
            and quantity of nodes to remove
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        resp = cluster.scale_in_by_node_type(self, nodes)
        return resp

    def replace_cluster_node(self, cluster, node_id):
        """Replace a failed node

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param node_id: ID of the node to be replaced
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.replace_node(self, node_id)

    def add_cluster_nodes(
        self, cluster, node_type, flavor, node_size, volume_type
    ):
        """Add master and client nodes to a cluster

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param node_type: node type
        :param flavor: flavor ID
        :param node_size: number of nodes
        :param volume_type: node storage type
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.add_nodes(
            self, node_type, flavor, node_size, volume_type
        )

    def get_cluster_upgrade_status(self, cluster, **params):
        """Obtain the cluster updgrade details

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
            return: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._list(
            _cluster_upgrade_status.ClusterUpgradeStatus,
            cluster_id=cluster.id,
            **params,
        )

    def retry_cluster_upgrade_job(self, cluster, job_id, retry_mode=None):
        """Retry a task or terminate the impact of a task

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param job_id: ID of the task to be retried
        :param retry_mode: if this parameter is not left blank, the impact
            of the task is terminated
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return cluster.retry_upgrade_job(self, job_id, retry_mode)

    # ======== Flavors ========
    def flavors(self):
        """List all Flavors.

        :returns: a generator of
            (:class:`~otcextensions.sdk.css.v1.flavor.Flavor`) instances
        """
        return self._list(_flavor.Flavor, paginated=False)

    # ======== Snapshot ========
    def snapshots(self, cluster):
        """List all Snapshots of a cluster.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :returns: a generator of
            (:class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`) instances
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        base_path = f'/clusters/{cluster.id}/index_snapshots'
        return self._list(_snapshot.Snapshot, base_path=base_path)

    def create_snapshot(self, cluster, **attrs):
        """Create a cluster Snapshot from attributes

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`,
            comprised of the properties on the Snapshot class.
        :returns: The results of cluster snapshot creation
        :rtype: :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._create(
            _snapshot.Snapshot, uri_cluster_id=cluster.id, **attrs
        )

    def find_snapshot(self, cluster, name_or_id, ignore_missing=True):
        """Find a single snapshot

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param name_or_id: The name or ID of a snapshot
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            if the snapshot does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent snapshot.

        :returns:
            One :class:`~otcextensions.sdk.dws.v1.snapshot.Snapshot` or
            ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._find(
            _snapshot.Snapshot,
            name_or_id,
            base_path=f'/clusters/{cluster.id}/index_snapshots',
            ignore_missing=ignore_missing,
        )

    def delete_snapshot(self, cluster, snapshot, ignore_missing=True):
        """Delete a snapshot

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param snapshot: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the snapshot does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent snapshot.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._delete(
            _snapshot.Snapshot,
            snapshot,
            uri_cluster_id=cluster.id,
            ignore_missing=ignore_missing,
        )

    def set_snapshot_configuration(
        self, cluster, auto_configure=False, **attrs
    ):
        """Setting Basic Configurations of a Cluster Snapshot

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param bool auto_configure: When set to ``True``
            Basic Configurations of a Cluster Snapshot will be set
            automatically.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`,
            comprised of the properties on the Snapshot class.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        setting = 'setting'
        if auto_configure and isinstance(auto_configure, bool):
            setting = 'auto_setting'
            attrs = {'has_body': False}
        return self._create(
            _snapshot.SnapshotConfiguration,
            cluster_id=cluster.id,
            setting=setting,
            **attrs,
        )

    def disable_snapshot_function(self, cluster):
        """Disable the snapshot function of a cluster.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        snapshot_config = self._get_resource(
            _snapshot.SnapshotConfiguration, '', cluster_id=cluster.id
        )
        snapshot_config.disable(self)

    def set_snapshot_policy(self, cluster, **attrs):
        """Set parameters related to automatic snapshot creation.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`,
            comprised of the properties on the Snapshot class.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._create(
            _snapshot.SnapshotPolicy, cluster_id=cluster.id, **attrs
        )

    def get_snapshot_policy(self, cluster):
        """Query the automatic snapshot creation policy for a cluster.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :returns: instance of
            :class:`~otcextensions.sdk.css.v1.snapshot.SnapshotPolicy`
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        return self._get(
            _snapshot.SnapshotPolicy, cluster_id=cluster.id, requires_id=False
        )

    def restore_snapshot(self, cluster, snapshot, **attrs):
        """Restore a snapshot.

        :param cluster: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.cluster.Cluster`
        :param snapshot: key id or an instance of
            :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.css.v1.snapshot.Snapshot`,
            comprised of the properties on the Snapshot class.
        :returns: ``None``
        """
        cluster = self._get_resource(_cluster.Cluster, cluster)
        snapshot = self._get_resource(_snapshot.Snapshot, snapshot)
        return snapshot.restore(self, cluster.id, **attrs)

    def download_certificate(self, filename=None):
        """
        Downloads a security certificate for ssl connectivity.
        The certificate is downloaded and saved to the specified file.

        :param filename: The name of the file to save the certificate as.
            If not provided, the default filename 'CloudSearchService.cer'
            is used.

        :returns: ``None``
        """
        headers = {'Accept': '*/*'}
        response = self.get('/cer/download', headers=headers, stream=True)
        exceptions.raise_from_response(response)

        # Extract the filename from Content-Disposition header if available
        content_disposition = response.headers.get('Content-Disposition', '')

        override_filename = filename
        filename = override_filename or 'CloudSearchService.cer'
        if not override_filename and 'filename=' in content_disposition:
            filename = content_disposition.split('filename=')[1].strip('"')

        if os.path.exists(filename):
            raise FileExistsError(
                f"The file '{filename}' already exists. Aborting download."
            )

        with open(filename, 'wb') as f:
            for chunk in response.iter_content(chunk_size=1024):
                if chunk:
                    f.write(chunk)

    def wait_for_cluster(
        self, cluster, timeout=1200, wait=5, print_status=False
    ):
        org_timeout = timeout
        while timeout > 0:
            obj = self.get_cluster(cluster)
            if getattr(obj, 'error'):
                raise exceptions.SDKException(obj.error)
            if obj.status_code == 100:
                pass
            elif obj.actions == [] and obj.action_progress == {}:
                return True
            self.log.debug(
                'Still waiting for resource %s to reach state %s, '
                'current state is %s'
                '\nWait Time Out remaining: %s seconds.',
                obj.name,
                'AVAILABLE',
                str(obj.action_progress or obj.status),
                str(timeout),
            )
            if print_status:
                dots = '.' * round(100 - ((timeout / org_timeout) * 100))
                print(
                    'CSS Cluster progress: '
                    + str(obj.action_progress or obj.status)
                    + ' '
                    + dots,
                    end='\r',
                )
            time.sleep(wait)
            timeout = timeout - wait
        raise exceptions.SDKException(
            'Wait Timed Out. Cluster action still in progress.'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import vpc_channel

EXAMPLE_MEMBER_GROUP = {
    'member_group_name': 'group1',
    'member_group_remark': 'test',
    'member_group_weight': 10,
    'dict_code': 'code',
    'microservice_version': '1.0',
    'microservice_port': 8080,
    'microservice_labels': [{'label_name': 'env', 'label_value': 'prod'}],
    'member_group_id': 'mg-123',
    'create_time': '2025-01-01T12:00:00Z',
    'update_time': '2025-01-02T12:00:00Z'
}

EXAMPLE_MEMBER = {
    'host': '192.168.0.1',
    'weight': 20,
    'is_backup': True,
    'member_group_name': 'group1',
    'status': 1,
    'port': 80,
    'ecs_id': 'ecs-123',
    'ecs_name': 'ecs-node'
}

EXAMPLE_HEALTH_CONFIG = {
    'protocol': 'HTTP',
    'path': '/health',
    'method': 'GET',
    'port': 8080,
    'threshold_normal': 3,
    'threshold_abnormal': 2,
    'time_interval': 5,
    'http_code': '200',
    'enable_client_ssl': False,
    'status': 1,
    'timeout': 2,
    'vpc_channel_id': 'vpc-123',
    'id': 'hc-123',
    'create_time': '2025-01-01T12:00:00Z'
}

EXAMPLE_MICROSERVICE = {
    'service_type': 'CSE',
    'cse_info': {
        'engine_id': 'e1',
        'service_id': 's1',
        'engine_name': 'engine',
        'service_name': 'service',
        'register_address': '127.0.0.1',
        'cse_app_id': 'app',
        'version': '1.0'
    },
    'cce_info': {
        'cluster_id': 'c1',
        'namespace': 'ns',
        'workload_type': 'deployment',
        'app_name': 'app',
        'label_key': 'env',
        'label_value': 'prod',
        'cluster_name': 'cname'
    },
    'id': 'ms-1',
    'instance_id': 'i-1',
    'create_time': '2025-01-01T12:00:00Z',
    'update_time': '2025-01-02T12:00:00Z'
}

EXAMPLE_VPC_CHANNEL = {
    'gateway_id': 'gw-1',
    'id': 'vpc-123',
    'name': 'test-channel',
    'port': 443,
    'balance_strategy': 1,
    'member_type': 'IP',
    'type': 1,
    'dict_code': 'code',
    'member_groups': [EXAMPLE_MEMBER_GROUP],
    'members': [EXAMPLE_MEMBER],
    'vpc_health_config': EXAMPLE_HEALTH_CONFIG,
    'microservice_info': EXAMPLE_MICROSERVICE,
    'create_time': '2025-01-01T12:00:00Z',
    'status': 1
}


class TestVpcChannel(base.TestCase):

    def test_basic(self):
        sot = vpc_channel.VpcChannel()
        self.assertEqual('apigw/instances/%(gateway_id)s/vpc-channels',
                         sot.base_path)
        self.assertEqual('vpc_channels', sot.resources_key)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = vpc_channel.VpcChannel(**EXAMPLE_VPC_CHANNEL)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['id'], sot.id)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['name'], sot.name)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['port'], sot.port)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['status'], sot.status)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['balance_strategy'],
                         sot.balance_strategy)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['member_type'], sot.member_type)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['type'], sot.type)
        self.assertEqual(EXAMPLE_VPC_CHANNEL['dict_code'], sot.dict_code)

        self.assertEqual(1, len(sot.member_groups))
        self.assertEqual('group1', sot.member_groups[0].member_group_name)
        self.assertEqual(1, len(sot.members))
        self.assertEqual('192.168.0.1', sot.members[0].host)
        self.assertEqual('HTTP', sot.vpc_health_config.protocol)
        self.assertEqual('CSE', sot.microservice_info.service_type)
        self.assertEqual('e1', sot.microservice_info.cse_info.engine_id)
        self.assertEqual('c1', sot.microservice_info.cce_info.cluster_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource
from openstack import utils
from otcextensions.sdk.modelartsv2.v2.dataset import DataSourceSpec


class SampleMetadataSpec(resource.Resource):
    #: Whether the sample is labeled as a hard sample, which is a default
    #:  attribute.
    hard = resource.Body("@modelarts:hard", type=float)
    #: Coefficient of difficulty of each sample level, which is a default
    #:  attribute.
    hard_coefficient = resource.Body("@modelarts:hard_coefficient", type=float)
    #: ID of a hard sample reason, which is a default attribute.
    hard_reasons = resource.Body(
        "@modelarts:hard_reasons", type=list, list_type=int
    )
    #: Image size (width, height, and depth of the image), which is a default
    #:  attribute, with type of List.
    size = resource.Body("@modelarts:size", type=list, list_type=int)


class SampleLabelPropertySpec(resource.Resource):
    #: Speech text content, which is a default attribute dedicated to the
    #:  speech label (including the speech content and speech start and end
    #:  points).
    content = resource.Body("@modelarts:content")
    #: End position of the text, which is a default attribute dedicated to the
    #:  named entity label.
    end_index = resource.Body("@modelarts:end_index", type=int)
    #: Speech end time, which is a default attribute dedicated to the speech
    #:  start/end point label, in the format of hh:mm:ss.
    end_time = resource.Body("@modelarts:end_time")
    #: Shape feature, which is a default attribute dedicated to the object
    #:  detection label, with type of List.
    feature = resource.Body("@modelarts:feature", type=dict)
    #: ID of the head entity in the triplet relationship label, which is a
    #:  default attribute dedicated to the triplet relationship label.
    from_id = resource.Body("@modelarts:from")
    #: Sample labeled as a hard sample or not, which is a default attribute.
    hard = resource.Body("@modelarts:hard")
    #: Coefficient of difficulty of each label level, which is a default
    #:  attribute.
    hard_coefficient = resource.Body("@modelarts:hard_coefficient")
    #: Reasons that the sample is a hard sample, which is a default attribute.
    hard_reasons = resource.Body("@modelarts:hard_reasons")
    #: Object shape, which is a default attribute dedicated to the object
    #:  detection label and is left empty by default.
    shape = resource.Body("@modelarts:shape")
    #: Speech source, which is a default attribute dedicated to the speech
    #:  start/end point label and can be set to a speaker or narrator.
    source = resource.Body("@modelarts:source")
    #: Start position of the text, which is a default attribute dedicated to
    #:  the named entity label.
    start_index = resource.Body("@modelarts:start_index", type=int)
    #: Speech start time, which is a default attribute dedicated to the speech
    #:  start/end point label, in the format of hh:mm:ss.
    start_time = resource.Body("@modelarts:start_time")
    #: ID of the tail entity in the triplet relationship label, which is a
    #:  default attribute dedicated to the triplet relationship label.
    to = resource.Body("@modelarts:to")


class SampleLabelSpec(resource.Resource):
    #: Video labeling method, which is used to distinguish whether a video is
    #:  labeled manually or automatically.
    annotated_by = resource.Body("annotated_by")
    #: Label ID.
    id = resource.Body("id")
    #: Label name.
    name = resource.Body("name")
    #: Attribute key-value pair of the sample label, such as the object shape
    #:  and shape feature.
    property = resource.Body("property", type=SampleLabelPropertySpec)
    #: Confidence.
    score = resource.Body("score", type=float)
    #: Label type.
    type = resource.Body("type", type=int)


class WorkerSpec(resource.Resource):
    #: Creation time.
    create_time = resource.Body("create_time", type=int)
    #: Labeling team member description.
    description = resource.Body("description")
    #: Email address of a labeling team member.
    email = resource.Body("email")
    #: Role.
    role = resource.Body("role", type=int)
    #: Current login status of a labeling team member.
    status = resource.Body("status", type=int)
    #: Update time.
    update_time = resource.Body("update_time", type=int)
    #: ID of a labeling team member.
    worker_id = resource.Body("worker_id")
    #: ID of a labeling team.
    workforce_id = resource.Body("workforce_id")


class HardDetailSpec(resource.Resource):
    #: Alias.
    alias_name = resource.Body("alo_name")
    #: Reason ID.
    id = resource.Body("id", type=int)
    #: Reason description.
    reason = resource.Body("reason")
    #: Handling suggestion.
    suggestion = resource.Body("suggestion")


class LabelFormatSpec(resource.Resource):
    #: Label type of text classification.
    label_type = resource.Body("label_type")
    #: Separator between labels.
    text_label_separator = resource.Body("text_label_separator")
    #: Separator between the text and label.
    text_sample_separator = resource.Body("text_sample_separator")


class SampleSpec(resource.Resource):
    # Video labeling method, which is used to distinguish
    #:  whether a video is labeled manually or automatically.
    annotated_by = resource.Body("annotated_by")
    #: Whether the acceptance is passed, which is used for team labeling.
    check_accept = resource.Body("check_accept", type=bool)
    #: Acceptance comment, which is used for team labeling.
    check_comment = resource.Body("check_comment")
    #: Acceptance score, which is used for team labeling.
    check_score = resource.Body("check_score")
    #: Byte data of sample files.
    data = resource.Body("data", type=dict)
    #: Data source.
    data_source = resource.Body("data_source", type=DataSourceSpec)
    #: Reason for deleting a sample, which is used for healthcare.
    deletion_reasons = resource.Body("deletion_reasons", type=list)
    #: Encoding type of sample files, which is used to upload .
    encoding = resource.Body("encoding")
    #: Check the document. Details about difficulties, including
    #:  description, causes, and suggestions of difficult problems.
    hard_details = resource.Body("hard_details")
    #: Labeling personnel list of sample assignment.
    labelers = resource.Body("labelers", type=list, list_type=WorkerSpec)
    #: Sample label list.
    labels = resource.Body("labels", type=list, list_type=SampleLabelSpec)
    #: Key-value pair of the sample metadata attribute.
    metadata = resource.Body("metadata", type=SampleMetadataSpec)
    #: Name of sample files.
    name = resource.Body("name")
    #: Preview dataset sample.
    preview = resource.Body("preview")
    #: Whether to accept the review, which is used for team labeling.
    review_accept = resource.Body("review_accept", type=bool)
    #: Review comment, which is used for team labeling.
    review_comment = resource.Body("review_comment")
    #: Review score, which is used for team labeling.
    review_score = resource.Body("review_score")
    #: Sample data list.
    sample_data = resource.Body("sample_data", type=list)
    #: Sample path.
    sample_dir = resource.Body("sample_dir")
    #: Sample ID.
    sample_id = resource.Body("sample_id")
    #: Sample name.
    sample_name = resource.Body("sample_name")
    #: Sample size or text length, in bytes.
    sample_size = resource.Body("sample_size", type=int)
    #: Sample status.
    sample_status = resource.Body("sample_status")
    #: Sample time, when OBS is last modified.
    sample_time = resource.Body("sample_time", type=int)
    #: Sample type.
    sample_type = resource.Body("sample_type", type=int)
    #: Comprehensive score, which is used for team labeling.
    score = resource.Body("score")
    #: Source address of sample data.
    source = resource.Body("source")
    #: Subsample URL, which is used for healthcare.
    sub_sample_url = resource.Body("sub_sample_url")
    #: ID of a labeling team member, which is used for team labeling.
    worker_id = resource.Body("worker_id")


class SampleRespSpec(resource.Resource):
    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Description.
    info = resource.Body("info")
    #: Name of a sample file.
    name = resource.Body("name")
    #: Whether the operation is successful.
    success = resource.Body("success", type=bool)


class DatasetSample(SampleSpec):
    base_path = "/datasets/%(dataset_id)s/data-annotations/samples"

    resources_key = "samples"

    _query_mapping = resource.QueryParameters(
        "email",
        "high_score",
        "label_name",
        "label_type",
        "limit",
        "locale",
        "low_score",
        "offset",
        "order",
        "preview",
        "process_parameter",
        "sample_state",
        "sample_type",
        "search_conditions",
        "version_id",
    )

    # Capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True

    # Properties
    #: Dataset ID.
    dataset_id = resource.URI("dataset_id")
    #: Whether to directly import to the final result.
    final_annotation = resource.Body("final_annotation", type=bool)
    #: Label format.
    label_format = resource.Body("label_format", type=LabelFormatSpec)
    #: Sample list.
    samples = resource.Body("samples", type=list, list_type=SampleSpec)
    #: Sample ID.
    sample_id = resource.Body("sample_id", alternate_id=True)

    #: Error code.
    error_code = resource.Body("error_code")
    #: Error message.
    error_msg = resource.Body("error_msg")
    #: Response list for adding samples in batches.
    results = resource.Body("results", type=list, list_type=SampleRespSpec)
    #: Whether the operation is successful.
    success = resource.Body("success", type=bool)

    def delete_samples(
        self, session, dataset_id, samples=[], delete_source=False
    ):
        """Delete dataset samples"""
        url = utils.urljoin(
            self.base_path % {"dataset_id": dataset_id}, "delete"
        )
        json_body = {"samples": samples}
        if delete_source:
            json_body.update(delete_source=True)
        response = session.post(url, json=json_body)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Share member CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_member(obj):
    """Flatten the structure of the member into a single dict
    """
    data = {
        'id': obj.id,
        'status': obj.status,
        'created_at': obj.created_at,
        'updated_at': obj.updated_at,
        'backup_id': obj.backup_id,
        'image_id': obj.image_id,
        'dest_project_id': obj.dest_project_id,
        'vault_id': obj.vault_id
    }

    return data


def _add_members_to_obj(obj, data, columns):
    """Add members to obj.members
    """
    i = 0
    for member in obj.members:
        name = 'member_' + str(i + 1)
        data += ('\n'.join((f'{a}={v}' for a, v in member[i])),)
        columns = columns + (name, )
        i += 1
    return data, columns


class ListMembers(command.Lister):

    _description = _('List CBR Share Members')
    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    def get_parser(self, prog_name):
        parser = super(ListMembers, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('Backup name or id.')
        )
        parser.add_argument(
            '--dest-project-id',
            metavar='<dest_project_id>',
            help=_('ID of the project with which the backup is shared.')
        )
        parser.add_argument(
            '--image-id',
            metavar='<image_id>',
            help=_('ID of the image created from the accepted backup.')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Number of records displayed per page.'
                   ' The value must be a positive integer.')
        )
        parser.add_argument(
            '--marker',
            metavar='<marker>',
            help=_('ID of the last record displayed on the previous page.'
                   ' Only UUID is supported.')
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_('Offset value, which is a positive integer.')
        )
        parser.add_argument(
            '--sort',
            metavar='<sort>',
            choices=['acs', 'desc', 'sort'],
            help=_('A group of properties separated by commas (,)'
                   ' and sorting directions.')
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            help=_('Status of a shared backup.')
        )
        parser.add_argument(
            '--vault-id',
            metavar='<vault_id>',
            help=_('ID of the vault where the shared backup is stored. '
                   'Only UUID is supported.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        query = {}

        if parsed_args.dest_project_id:
            query['dest_project_id'] = parsed_args.dest_project_id
        if parsed_args.image_id:
            query['image_id'] = parsed_args.image_id
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.marker:
            query['marker'] = parsed_args.marker
        if parsed_args.offset:
            query['offset'] = parsed_args.offset
        if parsed_args.sort:
            query['sort'] = parsed_args.sort
        if parsed_args.status:
            query['status'] = parsed_args.status
        if parsed_args.vault_id:
            query['vault_id'] = parsed_args.vault_id

        backup = client.find_backup(
            name_or_id=parsed_args.backup, ignore_missing=False)

        members_data = client.members(backup=backup.id, **query)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_member(s), self.columns,
                 ) for s in members_data))
        return table


class ShowMember(command.ShowOne):
    _description = _('Show single CBR Share Member.')
    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    def get_parser(self, prog_name):
        parser = super(ShowMember, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('Backup name or id.')
        )
        parser.add_argument(
            'member',
            metavar='<member_id>',
            help=_('Member ID, which is the project ID of the tenant'
                   ' who receives the shared backup.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        backup = client.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )
        obj = client.get_member(
            member=parsed_args.member,
            backup=backup.id
        )

        data = utils.get_dict_properties(
            _flatten_member(obj), self.columns)

        return (self.columns, data)


class AddMembers(command.Lister):
    _description = _('Add Share Member.')
    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    def get_parser(self, prog_name):
        parser = super(AddMembers, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('Backup name or id.')
        )
        parser.add_argument(
            '--members',
            action='append',
            required=True,
            help=_('Project IDs of the backup share members to be added.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        client = self.app.client_manager.cbr
        backup = client.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )

        attrs['members'] = parsed_args.members

        client = self.app.client_manager.cbr
        members_data = client.add_members(backup=backup.id,
                                          members=attrs['members'])

        data = (utils.get_dict_properties(_flatten_member(s),
                                          self.columns,) for s in members_data)
        return (self.columns, data)


class UpdateMember(command.ShowOne):
    _description = _('Update the Share Member Status.')
    columns = ('id', 'status', 'created_at', 'updated_at', 'backup_id',
               'image_id', 'dest_project_id', 'vault_id')

    def get_parser(self, prog_name):
        parser = super(UpdateMember, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            metavar='<backup>',
            help=_('Backup name or id.')
        )
        parser.add_argument(
            'member',
            metavar='<member_id>',
            help=_('Member ID, which is the same ID as in project ID.')
        )
        parser.add_argument(
            '--status',
            metavar='status',
            required=True,
            choices=['accepted', 'pending', 'rejected'],
            help=_('Status of a shared backup.')
        )
        parser.add_argument(
            '--vault-id',
            metavar='vault_id',
            help=_('Specifies the vault in which the shared backup is'
                   ' to be stored. Only UUID is supported. When updating'
                   ' the status of a backup share member status,'
                   ' if the backup is accepted, vault_id must be specified.'
                   ' If the backup is rejected, vault_id is not required.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        client = self.app.client_manager.cbr

        attrs['status'] = parsed_args.status

        if parsed_args.vault_id:
            attrs['vault'] = parsed_args.vault_id

        if attrs:
            obj = client.update_member(backup=parsed_args.backup,
                                       member=parsed_args.member, **attrs)

            data = utils.get_dict_properties(
                _flatten_member(obj), self.columns)

            return (self.columns, data)


class DeleteMember(command.Command):
    _description = _('Delete Share Member')

    def get_parser(self, prog_name):
        parser = super(DeleteMember, self).get_parser(prog_name)
        parser.add_argument(
            'backup',
            help=_('Backup name or id.')
        )
        parser.add_argument(
            'member',
            help=_('Member id.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        backup = client.find_backup(
            name_or_id=parsed_args.backup,
            ignore_missing=False
        )

        self.app.client_manager.cbr.delete_member(
            backup=backup.id, member=parsed_args.member,
            ignore_missing=False)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function
from otcextensions.sdk.function_graph.v2 import version
from otcextensions.sdk.function_graph.v2 import alias

from openstack import _log

from otcextensions.tests.functional.sdk.function_graph import TestFg

_logger = _log.setup_logging('openstack')


class TestFunctionAlias(TestFg):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionAlias, self).setUp()
        self.function = self.client.create_function(**TestFg.function_attrs)
        assert isinstance(self.function, function.Function)

        self.publish_attrs = {
            'version': 'new-version',
            'description': 'otce',
        }

        self.publish = self.client.publish_version(
            self.function, **self.publish_attrs
        )
        assert isinstance(self.publish, version.Version)

        self.alias_attrs = {
            'name': 'a1',
            'version': 'new-version'
        }
        self.alias = self.client.create_alias(
            self.publish.func_urn, **self.alias_attrs
        )
        assert isinstance(self.alias, alias.Alias)
        self.ID = self.alias.name
        self.addCleanup(
            self.client.delete_function,
            self.function
        )
        self.addCleanup(
            self.client.delete_alias,
            self.function,
            self.alias
        )

    def test_function_aliases(self):
        elist = list(self.client.aliases(
            func_urn=self.function.func_urn))
        self.assertIn(self.alias_attrs['name'], elist[0].name)

    def test_function_published_versions(self):
        vlist = list(self.client.versions(
            func_urn=self.function.func_urn))
        self.assertIn(self.publish_attrs['version'], vlist[1].version)

    def test_get_function_alias(self):
        a = self.client.get_alias(
            self.function, self.alias)
        self.assertIn(self.ID, a.id)

    def test_update_function_alias(self):
        attrs = {
            'version': 'new-version',
            'description': 'new',
        }
        updated = self.client.update_alias(
            self.function, self.alias, **attrs)
        self.assertIn(attrs['version'], updated.version)
        self.assertIn(attrs['description'], updated.description)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""
Create DeH instance.

For a full guide see
http://developer.openstack.org/sdks/python/openstacksdk/user/guides/deh.html
"""


def create_host(conn):
    print("Create Host:")

    host = conn.deh.create_host(
        name='my_new_host',
        # host_type can be retrieved with host_types() call
        host_type='general',
        availability_zone='eu-de-01',
        # amount of hosts to allocate
        quantity=1
    )

    # dedicated_host_ids is a list of allocated hosts (since multiple may
    # be requested)

    print(host.dedicated_host_ids)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Node Pool CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_node_pool(obj):
    """Flatten the structure of the node pool into a single dict
    """
    sdct = obj.spec.autoscaling.scale_down_cooldown_time
    nw = obj.spec.node_template_spec.node_nic_spec.primary_nic.network_id
    data = {
        'id': obj.id,
        'name': obj.name,
        'flavor': obj.spec.node_template_spec.flavor,
        'os': obj.spec.node_template_spec.os,
        'current_node': obj.status.current_node,
        'network_id': nw,
        'root_volume_type': obj.spec.node_template_spec.root_volume.type,
        'root_volume_size': obj.spec.node_template_spec.root_volume.size,
        'data_volume_type': obj.spec.node_template_spec.data_volumes[0].type,
        'data_volume_size': obj.spec.node_template_spec.data_volumes[0].size,
        'autoscaling': obj.spec.autoscaling.enable,
        'min_node_count': obj.spec.autoscaling.min_node_count,
        'max_node_count': obj.spec.autoscaling.max_node_count,
        'scale_down_cooldown_time': sdct,
        'priority': obj.spec.autoscaling.priority,
        'status': obj.status.status,
    }

    return data


class ListCCENodePools(command.Lister):
    _description = _('List CCE Node Pools')
    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'autoscaling',
        'current_node',
        'status')

    def get_parser(self, prog_name):
        parser = super(ListCCENodePools, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The ID or name of the cluster.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        cluster = client.find_cluster(parsed_args.cluster,
                                      ignore_missing=False)

        data = client.node_pools(cluster.id)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_node_pool(s), self.columns,
                 ) for s in data))
        return table


class ShowCCENodePool(command.ShowOne):
    _description = _('Show single Cluster node details')
    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'current_node',
        'network_id',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type',
        'data_volume_size',
        'autoscaling',
        'min_node_count',
        'max_node_count',
        'scale_down_cooldown_time',
        'priority',
        'status')

    def get_parser(self, prog_name):
        parser = super(ShowCCENodePool, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'nodepool',
            metavar='<nodepool>',
            help=_('ID of the CCE node pool.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        cluster = client.find_cluster(parsed_args.cluster,
                                      ignore_missing=False)

        obj = client.find_node_pool(
            cluster=cluster.id,
            node_pool=parsed_args.nodepool
        )

        data = utils.get_dict_properties(
            _flatten_node_pool(obj), self.columns)

        return (self.columns, data)


class CreateCCENodePool(command.ShowOne):
    _description = _('Create CCE Node Pool')
    columns = (
        'ID',
        'name',
        'flavor',
        'os',
        'current_node',
        'network_id',
        'root_volume_type',
        'root_volume_size',
        'data_volume_type',
        'data_volume_size',
        'autoscaling',
        'min_node_count',
        'max_node_count',
        'scale_down_cooldown_time',
        'priority',)

    def get_parser(self, prog_name):
        parser = super(CreateCCENodePool, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the CCE cluster.')
        )
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name of the CCE Node Pool.')
        )
        parser.add_argument(
            '--autoscaling-enabled',
            metavar='<autoscaling_enabled>',
            default=False,
            type=bool,
            help=_('Enables or disables Autoscaling for cluster nodes.')
        )
        parser.add_argument(
            '--az',
            metavar='<availability_zone>',
            default='random',
            help=_('Availability zone for cluster nodes.\n'
                   'Default: random')
        )
        parser.add_argument(
            '--data-volume',
            metavar='volumetype=<volumetype>,size=<disksize>,'
                    'encrypted=<True|False>,cmk_id=<cmk_id>',
            action=parseractions.MultiKeyValueAction,
            dest='data_volumes',
            required_keys=['volumetype', 'size'],
            optional_keys=['encrypted', 'cmk_id'],
            help=_('Example: '
                   '--data-volume volumetype=SATA,size=100,encrypted=True,'
                   'cmk_id=12345qwertz \n'
                   'Repeat option for multiple data volumes.\n'
                   'Default: --data-volume volumetype=SATA,size=100')
        )
        parser.add_argument(
            '--ecs-group',
            metavar='<ecs_group>',
            help=_('ECS group id of the ECS group to which nodes belong '
                   'after creation.')
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('CCE cluster node flavor.')
        )
        parser.add_argument(
            '--initial-node-count',
            metavar='<initial_node_count>',
            default=0,
            type=int,
            help=_('Node count after Node pool creation.')
        )
        parser.add_argument(
            '--k8s-tag',
            metavar='<key_name>=<value_name>',
            action=parseractions.KeyValueAction,
            dest='k8s_tags',
            help=_('Kubernetes tags in form of key, value pairs. Repeat '
                   'option for multiple tags.\n'
                   'Example: '
                   '--k8s-tag keyname1=valuename1 '
                   '--k8s-tag keyname2=valuename2')
        )
        parser.add_argument(
            '--lvm-config',
            metavar='<lvm_config>',
            help=_('Config Map of the Docker data disk.')
        )
        parser.add_argument(
            '--min-node-count',
            metavar='<min_node_count>',
            type=int,
            help=_('If auto-scaling is enabled, the value describes the '
                   'minimum number of nodes of the cluster.')
        )
        parser.add_argument(
            '--max-node-count',
            metavar='<max_node_count>',
            type=int,
            help=_('If auto-scaling is enabled, the value describes the '
                   'maximum number of nodes of the cluster. The value '
                   'must be equal or greater as the min_node_count.')
        )
        parser.add_argument(
            '--max-pods',
            metavar='<max_pods>',
            type=int,
            help=_('Maximum number of pods on the node.')
        )
        parser.add_argument(
            '--node-image-id',
            metavar='<node_image_id>',
            help=_('Mandatory if custom image is used on a bare '
                   'metall node.')
        )
        parser.add_argument(
            '--node-pool-type',
            metavar='<node_pool_type>',
            help=_('Node pool type, currently only vm is supported.')
        )
        parser.add_argument(
            '--os',
            metavar='<operating_system>',
            required=True,
            help=_('CCE cluster node operating system.')
        )
        parser.add_argument(
            '--preinstall-script',
            metavar='<preinstall_script>',
            help=_('Script required before installation. '
                   'Input must be Base64 encoded.')
        )
        parser.add_argument(
            '--postinstall-script',
            metavar='<postinstall_script>',
            help=_('Script required after installation. '
                   'Input must be Base64 encoded.')
        )
        parser.add_argument(
            '--priority',
            metavar='<priority>',
            type=int,
            help=_('If auto-scaling is enabled, the value describes the '
                   'priority of the scale up functionality between '
                   'several Node Pools. A higher priority number '
                   'indicates a higher priority starting from 1.')
        )
        parser.add_argument(
            '--public-key',
            metavar='<public_key>',
            help=_('Additional public key to be added to for login.')
        )
        parser.add_argument(
            '--network',
            metavar='<network>',
            required=True,
            help=_('ID or name of the network to which the cluster node '
                   'will belong to.')
        )
        parser.add_argument(
            '--root-volume-size',
            metavar='<root_volume_size>',
            type=int,
            default=40,
            help=_('Root volume size in GB.')
        )
        parser.add_argument(
            '--root-volume-type',
            metavar='<root_volume_type>',
            choices=['SAS', 'SATA', 'SSD'],
            default='SATA',
            help=_('Root volume type.\nOptions: SAS, SATA, SSD.')
        )
        parser.add_argument(
            '--scale-down-cooldown-time',
            metavar='<time in minutes>',
            type=int,
            help=_('Interval in minutes during which nodes added '
                   'after a scale-up will not be deleted.')
        )
        parser.add_argument(
            '--ssh-key',
            metavar='<ssh-key>',
            required=True,
            help=_('Name of the SSH public key.')
        )
        parser.add_argument(
            '--tag',
            metavar='key=<keyname1>,value=<value1>',
            action=parseractions.MultiKeyValueAction,
            dest='tags',
            required_keys=['key', 'value'],
            help=_('Example: '
                   '--tag key=mykey1,value=myvalue1\n'
                   'Repeat option for multiple tags.')
        )
        parser.add_argument(
            '--taint',
            metavar='key=<keyname1>,value=<value1>,\n'
                    'effect=<NoSchedule|PreferNoSchedule|NoExecute',
            action=parseractions.MultiKeyValueAction,
            dest='taints',
            required_keys=['key', 'value', 'effect'],
            help=_('Example: '
                   '--taint key=mykey1,value=myvalue1,effect=NoSchedule\n'
                   'Repeat option for multiple taints.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        # mandatory
        attrs['cluster'] = parsed_args.cluster
        attrs['flavor'] = parsed_args.flavor
        attrs['os'] = parsed_args.os
        attrs['name'] = parsed_args.name
        attrs['network'] = parsed_args.network
        attrs['ssh_key'] = parsed_args.ssh_key

        # optional
        if parsed_args.az:
            attrs['availability_zone'] = parsed_args.az
        if parsed_args.autoscaling_enabled:
            attrs['autoscaling_enabled'] = parsed_args.autoscaling_enabled
        if parsed_args.data_volumes:
            attrs['data_volumes'] = parsed_args.data_volumes
        if parsed_args.ecs_group:
            attrs['ecs_group'] = parsed_args.ecs_group
        if parsed_args.initial_node_count:
            attrs['initial_node_count'] = parsed_args.initial_node_count
        if parsed_args.k8s_tags:
            attrs['k8s_tags'] = parsed_args.k8s_tags
        if parsed_args.lvm_config:
            attrs['lvm_config'] = parsed_args.lvm_config
        if parsed_args.max_node_count:
            attrs['max_node_count'] = parsed_args.max_node_count
        if parsed_args.max_pods:
            attrs['max_pods'] = parsed_args.max_pods
        if parsed_args.min_node_count:
            attrs['min_node_count'] = parsed_args.min_node_count
        if parsed_args.node_image_id:
            attrs['node_image_id'] = parsed_args.node_image_id
        if parsed_args.node_pool_type:
            attrs['node_pool_type'] = parsed_args.node_pool_type
        if parsed_args.preinstall_script:
            attrs['preinstall_script'] = parsed_args.preinstall_script
        if parsed_args.postinstall_script:
            attrs['postinstall_script'] = parsed_args.postinstall_script
        if parsed_args.priority:
            attrs['priority'] = parsed_args.priority
        if parsed_args.public_key:
            attrs['public_key'] = parsed_args.public_key
        if parsed_args.root_volume_size:
            attrs['root_volume_size'] = parsed_args.root_volume_size
        if parsed_args.root_volume_type:
            attrs['root_volume_type'] = parsed_args.root_volume_type
        if parsed_args.scale_down_cooldown_time:
            sdct = parsed_args.scale_down_cooldown_time
            attrs['scale_down_cooldown_time'] = sdct
        if parsed_args.tags:
            attrs['tags'] = parsed_args.tags
        if parsed_args.taints:
            attrs['taints'] = parsed_args.taints

        obj = self.app.client_manager.sdk_connection.create_cce_node_pool(
            **attrs)

        data = utils.get_dict_properties(
            _flatten_node_pool(obj),
            self.columns)

        return (self.columns, data)


class DeleteCCENodePool(command.Command):
    _description = _('Delete CCE Node Pool')

    def get_parser(self, prog_name):
        parser = super(DeleteCCENodePool, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'nodepool',
            metavar='<nodepool>',
            nargs='+',
            help=_('Name or ID of CCE Node pool. Repeat names with spaces '
                   'in between to delete multiple node pools.\n'
                   'Example: '
                   'openstack cce node pool delete testcluster pool1 pool2')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.cluster and parsed_args.nodepool:
            client = self.app.client_manager.cce
            cluster = client.find_cluster(parsed_args.cluster,
                                          ignore_missing=False)
            for pool in parsed_args.nodepool:
                obj = client.find_node_pool(
                    cluster=cluster.id,
                    node_pool=pool)
                client.delete_node_pool(
                    cluster=cluster.id,
                    node_pool=obj.id,
                    ignore_missing=False)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create CCE Node Pool via OpenStack cloud layer
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')


pool = conn.create_cce_node_pool(
    autoscaling_enabled=False,
    availability_zone='random',
    billing_mode=0,
    cluster='bca73490-394c-11eb-8fe2-0255ac101123',
    count=0,
    flavor='s2.large.2',
    initial_node_count=0,
    k8s_tags={
        "muh": "kuh"},
    ssh_key='sshkey-pub',
    name='test',
    os='CentOS 7.7',
    root_volume_size=40,
    root_volume_type='SATA',
    network='25d24fc8-d019-4a34-9fff-0a09fde6a123',
    tags=[
        {
            'key': 'hellokey1',
            'value': 'hellovalue1'
        }, {
            'key': 'hellokey2',
            'value': 'hellovalue2'
        }],
    data_volumes=[
        {
            'volumetype': 'SATA',
            'size': 120,
        }, {
            'volumetype': 'SATA',
            'size': 100,
            'encrypted': True,
            'cmk_id': '5b16304e-aa0f-488e-9c9e-7d2402def123'
        }]
)
print(pool)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from openstackclient.tests.unit import utils

from otcextensions.osclient.volume_backup.v2 import policy
from otcextensions.tests.unit.osclient.volume_backup.v2 import fakes


class TestListPolicy(fakes.TestVolumeBackup):

    _objects = fakes.FakePolicy.create_multiple(3)

    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    data = []

    for s in _objects:
        data.append((
            s.id,
            s.name,
            s.policy_resource_count,
            policy.BackupPolicy(s.scheduled_policy),
            s.tags
        ))

    def setUp(self):
        super(TestListPolicy, self).setUp()

        self.cmd = policy.ListVolumeBackupPolicy(self.app, None)

        self.client.backup_policies = mock.Mock()

    def test_list_default(self):
        arglist = [
        ]

        verifylist = [
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.backup_policies.side_effect = [
            self._objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.backup_policies.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertListItemEqual(self.data, list(data))


class TestShowPolicy(fakes.TestVolumeBackup):

    _object = fakes.FakePolicy.create_one()

    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    data = (
        _object.id,
        _object.name,
        _object.policy_resource_count,
        policy.BackupPolicy(_object.scheduled_policy),
        _object.tags
    )

    def setUp(self):
        super(TestShowPolicy, self).setUp()

        self.cmd = policy.ShowVolumeBackupPolicy(self.app, None)

        self.client.find_backup_policy = mock.Mock()

    def test_show_default(self):
        arglist = [
            'policy_name',
        ]

        verifylist = [
            ('policy', 'policy_name'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_backup_policy.assert_called_once_with(
            name_or_id='policy_name',
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)


class TestCreatePolicy(fakes.TestVolumeBackup):

    _object = fakes.FakePolicy.create_one()

    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    data = (
        _object.id,
        _object.name,
        _object.policy_resource_count,
        policy.BackupPolicy(_object.scheduled_policy),
        _object.tags
    )

    def setUp(self):
        super(TestCreatePolicy, self).setUp()

        self.cmd = policy.CreateVolumeBackupPolicy(self.app, None)

        self.client.create_backup_policy = mock.Mock()

    def test_create_default(self):
        arglist = [
            'policy_name',
            '--start_time', '12:12',
            '--frequency', '13',
            '--rentention_num', '14',
            '--tag', 'a=b'
        ]

        verifylist = [
            ('name', 'policy_name'),
            ('start_time', '12:12'),
            ('enable', False),
            ('frequency', 13),
            ('rentention_num', 14),
            ('remain_first_backup_of_curMonth', False),
            ('tag', ['a=b']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_backup_policy.assert_called_once_with(
            name='policy_name',
            scheduled_policy={
                'start_time': '12:12',
                'frequency': 13,
                'rentention_num': 14,
                'remain_first_backup_of_curMonth': 'N',
                'status': 'OFF'
            },
            tags=[{'key': 'a', 'value': 'b'}]
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_optionals(self):
        arglist = [
            'policy_name',
            '--start_time', '12:12',
            '--enable',
            '--frequency', '13',
            '--rentention_num', '14',
            '--remain_first_backup_of_curMonth',
            '--tag', 'a=b',
            '--tag', 'c=d',
        ]

        verifylist = [
            ('name', 'policy_name'),
            ('start_time', '12:12'),
            ('enable', True),
            ('frequency', 13),
            ('rentention_num', 14),
            ('remain_first_backup_of_curMonth', True),
            ('tag', ['a=b', 'c=d']),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_backup_policy.assert_called_once_with(
            name='policy_name',
            scheduled_policy={
                'start_time': '12:12',
                'frequency': 13,
                'rentention_num': 14,
                'remain_first_backup_of_curMonth': 'Y',
                'status': 'ON'
            },
            tags=[{'key': 'a', 'value': 'b'}, {'key': 'c', 'value': 'd'}]
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_create_frequense_not_allowed(self):
        arglist = [
            'policy_name',
            '--start_time', '12:12',
            '--frequency', '15',
        ]

        verifylist = [
            ('name', 'policy_name'),
            ('start_time', '12:12'),
            ('enable', False),
            ('frequency', 15),
            ('remain_first_backup_of_curMonth', False),
        ]

        # Verify cm is raising exception due to the exclusive group
        self.assertRaises(
            utils.ParserException,
            self.check_parser,
            self.cmd, arglist, verifylist
        )


class TestUpdatePolicy(fakes.TestVolumeBackup):

    _object = fakes.FakePolicy.create_one()

    columns = ('id', 'name', 'policy_resource_count', 'scheduled_policy',
               'tags')

    data = (
        _object.id,
        _object.name,
        _object.policy_resource_count,
        policy.BackupPolicy(_object.scheduled_policy),
        _object.tags
    )

    def setUp(self):
        super(TestUpdatePolicy, self).setUp()

        self.cmd = policy.UpdateVolumeBackupPolicy(self.app, None)

        self.client.update_backup_policy = mock.Mock()

    def test_update_name(self):
        arglist = [
            'policy_id',
            '--name', 'policy_name',
            '--frequency', '13',
            '--rentention_num', '14',
        ]

        verifylist = [
            ('id', 'policy_id'),
            ('name', 'policy_name'),
            ('frequency', 13),
            ('rentention_num', 14),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_backup_policy.assert_called_once_with(
            backup_policy='policy_id',
            name='policy_name',
            scheduled_policy={
                'frequency': 13,
                'rentention_num': 14
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_update_disable_remain_first(self):
        arglist = [
            'policy_id',
            '--remain_first_backup_of_curMonth', 'false',
        ]

        verifylist = [
            ('id', 'policy_id'),
            ('remain_first_backup_of_curMonth', False),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_backup_policy.assert_called_once_with(
            backup_policy='policy_id',
            scheduled_policy={
                'remain_first_backup_of_curMonth': 'N'
            }
        )

        self.assertEqual(self.columns, columns)
        self.assertItemEqual(self.data, data)

    def test_update_enable_remain_first(self):
        arglist = [
            'policy_id',
            '--remain_first_backup_of_curMonth', '1',
        ]

        verifylist = [
            ('id', 'policy_id'),
            ('remain_first_backup_of_curMonth', True),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_backup_policy.assert_called_once_with(
            backup_policy='policy_id',
            scheduled_policy={
                'remain_first_backup_of_curMonth': 'Y'
            }
        )

    def test_update_disable(self):
        arglist = [
            'policy_id',
            '--status', 'false',
        ]

        verifylist = [
            ('id', 'policy_id'),
            ('status', False),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_backup_policy.assert_called_once_with(
            backup_policy='policy_id',
            scheduled_policy={
                'status': 'OFF'
            }
        )

    def test_update_enable(self):
        arglist = [
            'policy_id',
            '--status', 'True',
        ]

        verifylist = [
            ('id', 'policy_id'),
            ('status', True),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_backup_policy.side_effect = [
            self._object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.update_backup_policy.assert_called_once_with(
            backup_policy='policy_id',
            scheduled_policy={
                'status': 'ON'
            }
        )


class TestDeletePolicy(fakes.TestVolumeBackup):

    def setUp(self):
        super(TestDeletePolicy, self).setUp()

        self.cmd = policy.DeleteVolumeBackupPolicy(self.app, None)

        self.client.delete_backup_policy = mock.Mock()

    def test_delete_default(self):
        arglist = [
            'pol',
        ]

        verifylist = [
            ('policy', 'pol'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_backup_policy.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.delete_backup_policy.assert_called_once_with(
            backup_policy='pol',
            ignore_missing=False
        )


class TestExecutePolicy(fakes.TestVolumeBackup):

    def setUp(self):
        super(TestExecutePolicy, self).setUp()

        self.cmd = policy.ExecuteVolumeBackupPolicy(self.app, None)

        self.client.execute_policy = mock.Mock()

    def test_execute_default(self):
        arglist = [
            'pol',
        ]

        verifylist = [
            ('policy', 'pol'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.execute_policy.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.execute_policy.assert_called_once_with(
            backup_policy='pol'
        )


class TestLinkResourceToPolicy(fakes.TestVolumeBackup):

    def setUp(self):
        super(TestLinkResourceToPolicy, self).setUp()

        self.cmd = policy.LinkResourceToVolumeBackupPolicy(self.app, None)

        self.client.link_resources_to_policy = mock.Mock()

    def test_execute_default(self):
        arglist = [
            'pol',
            '--volume', 'v1',
            '--volume', 'v2'
        ]

        verifylist = [
            ('policy', 'pol'),
            ('volume', ['v1', 'v2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.link_resources_to_policy.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.link_resources_to_policy.assert_called_once_with(
            backup_policy='pol',
            resources=['v1', 'v2']
        )


class TestUnlinkResourceToPolicy(fakes.TestVolumeBackup):

    def setUp(self):
        super(TestUnlinkResourceToPolicy, self).setUp()

        self.cmd = policy.UnlinkResourceFromVolumeBackupPolicy(self.app, None)

        self.client.unlink_resources_of_policy = mock.Mock()

    def test_execute_default(self):
        arglist = [
            'pol',
            '--volume', 'v1',
            '--volume', 'v2'
        ]

        verifylist = [
            ('policy', 'pol'),
            ('volume', ['v1', 'v2'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.unlink_resources_of_policy.side_effect = [
            {}
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        self.client.unlink_resources_of_policy.assert_called_once_with(
            backup_policy='pol',
            resources=['v1', 'v2']
        )
